@@ -45,7 +45,7 @@
 
    This file implements a SANE backend for Abaton flatbed scanners.  */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <ctype.h>
 #include <errno.h>
@@ -61,28 +61,28 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include "_stdint.h"
+#include "../include/_stdint.h"
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_scsi.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
 
 #define BACKEND_NAME	abaton
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sanei_config.h"
+#include "../include/sane/sanei_config.h"
 #define ABATON_CONFIG_FILE "abaton.conf"
 
 #include "abaton.h"
 
 
-#define MM_PER_INCH	25.4
 
+static const SANE_Device **devlist = 0;
 static int num_devices;
 static Abaton_Device *first_dev;
 static Abaton_Scanner *first_handle;
@@ -343,11 +343,11 @@ set_window (Abaton_Scanner * s)
 
   invert = s->val[OPT_NEGATIVE].w;
 
-  if (!strcmp (s->val[OPT_MODE].s, "Lineart"))
+  if (!strcmp (s->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_LINEART))
     {
       STORE8 (window + 17, 0);
     }
-  else if (!strcmp (s->val[OPT_MODE].s, "Halftone"))
+  else if (!strcmp (s->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_HALFTONE))
     {
       STORE8 (window + 17, 1);
     }
@@ -514,7 +514,7 @@ calc_parameters (Abaton_Scanner * s)
   
   DBG (FLOW_CONTROL, "Entering calc_parameters\n");
 
-  if (!strcmp (val, "Lineart") || !strcmp (val, "Halftone"))
+  if (!strcmp (val, SANE_VALUE_SCAN_MODE_LINEART) || !strcmp (val, SANE_VALUE_SCAN_MODE_HALFTONE))
     {
       s->params.depth = 1;
       s->bpp = 1;
@@ -584,14 +584,14 @@ mode_update (SANE_Handle handle, char *val)
 {
   Abaton_Scanner *s = handle;
 
-  if (!strcmp (val, "Lineart"))
+  if (!strcmp (val, SANE_VALUE_SCAN_MODE_LINEART))
     {
       DISABLE (OPT_BRIGHTNESS);
       DISABLE (OPT_CONTRAST);
       ENABLE (OPT_THRESHOLD);
       DISABLE (OPT_HALFTONE_PATTERN);
     }
-  else if (!strcmp (val, "Halftone"))
+  else if (!strcmp (val, SANE_VALUE_SCAN_MODE_HALFTONE))
     {
       ENABLE (OPT_BRIGHTNESS);
       ENABLE (OPT_CONTRAST);
@@ -661,12 +661,12 @@ init_options (Abaton_Scanner * s)
   s->opt[OPT_MODE_GROUP].cap = 0;
   s->opt[OPT_MODE_GROUP].constraint_type = SANE_CONSTRAINT_NONE;
 
-  mode_list[0]="Lineart";
+  mode_list[0]=SANE_VALUE_SCAN_MODE_LINEART;
   
   switch (s->hw->ScannerModel)
     {
     case ABATON_300GS:
-      mode_list[1]="Halftone";
+      mode_list[1]=SANE_VALUE_SCAN_MODE_HALFTONE;
       mode_list[2]="Gray16";
       mode_list[3]="Gray256";
       mode_list[4]=NULL;
@@ -906,12 +906,14 @@ sane_exit (void)
       free ((void *) dev->sane.model);
       free (dev);
     }
+
+  if (devlist)
+    free (devlist);
 }
 
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
-  static const SANE_Device **devlist = 0;
   Abaton_Device *dev;
   int i;
 
@@ -1456,7 +1458,7 @@ sane_cancel (SANE_Handle handle)
       if (s->AbortedByUser)
 	{
 	  DBG (FLOW_CONTROL, "sane_cancel: Scan has not been initiated yet."
-	       "we probably recieved a signal while writing data.\n");
+	       "we probably received a signal while writing data.\n");
 	  s->AbortedByUser = SANE_FALSE;
 	}
       else
