@@ -51,6 +51,7 @@
 #undef Byte
 #define Byte SANE_Byte
 
+static const SANE_Device **devlist = 0;
 static int num_devices;
 static AgfaFocus_Device *agfafocus_devices;
 
@@ -946,7 +947,7 @@ do_cancel (AgfaFocus_Scanner * s)
 
   do_eof (s);
 
-  if (s->reader_pid != -1)
+  if (sanei_thread_is_valid (s->reader_pid))
     {
       int exit_status;
 
@@ -1323,12 +1324,14 @@ sane_exit (void)
 	sane_close (dev->handle);
       free (dev);
     }
+  
+  if (devlist)
+    free (devlist);
 }
 
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
-  static const SANE_Device **devlist = 0;
   AgfaFocus_Device *dev;
   int i;
 
@@ -2051,7 +2054,7 @@ sane_cancel (SANE_Handle handle)
 {
   AgfaFocus_Scanner *s = handle;
 
-  if (s->reader_pid != -1)
+  if (sanei_thread_is_valid (s->reader_pid))
     sanei_thread_kill (s->reader_pid);
   s->scanning = SANE_FALSE;
 }
