@@ -873,7 +873,6 @@ void
 sane_close (SANE_Handle handle)
 {
   Canon_Scanner *prev, *scanner;
-  SANE_Status res;
 
   DBG (3, "sane_close\n");
 
@@ -906,7 +905,7 @@ sane_close (SANE_Handle handle)
   else
     first_handle = scanner->next;
 
-  res = CANON_close_device (&scanner->scan);
+  CANON_close_device (&scanner->scan);
 
   free (scanner);
 }
