@@ -40,7 +40,7 @@
 
    -----
 
-   canon_pp.c: $Revision: 1.15 $
+   canon_pp.c: $Revision$
 
    This file is part of the canon_pp backend, supporting Canon FBX30P 
    and NX40P scanners
@@ -61,7 +61,7 @@
 #endif
 
 #ifndef VERSION
-#define VERSION "$Revision: 1.15 $"
+#define VERSION "$Revision$"
 #endif
 
 #include  <string.h>
@@ -150,12 +150,12 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 	DBG_INIT();
 
 #if defined PACKAGE && defined VERSION
-	DBG(2, ">> sane_init(%p, %p): " PACKAGE " " VERSION "\n", 
-			(const void*)vc, (const void*)cb);
+	DBG(2, ">> sane_init (version %s null, authorize %s null): " PACKAGE " " VERSION "\n", 
+	    (vc) ? "!=" : "==", (cb) ? "!=" : "==");
 #endif
 
 	if(vc)
-		*vc = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+		*vc = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
 	DBG(2,"sane_init: >> ieee1284_find_ports\n");
 	/* Find lp ports */
