@@ -181,6 +181,8 @@ typedef enum
 
   CS2_OPTION_INFRARED,
 
+  CS2_OPTION_SAMPLES_PER_SCAN,
+
   CS2_OPTION_DEPTH,
 
   CS2_OPTION_EXPOSURE,
@@ -249,8 +251,8 @@ typedef struct
 
   /* settings */
   SANE_Bool preview, negative, infrared;
-  int depth, real_depth, bytes_per_pixel, shift_bits, n_colour_in,
-    n_colour_out;
+  int samples_per_scan, depth, real_depth, bytes_per_pixel, shift_bits,
+	n_colour_in, n_colour_out;
   cs2_pixel_t n_lut;
   cs2_pixel_t *lut_r, *lut_g, *lut_b, *lut_neutral;
   unsigned long resx, resy, res, res_independent, res_preview;
@@ -508,6 +510,29 @@ sane_open (SANE_String_Const name, SANE_Handle * h)
 	  o.size = WSIZE;
 	  o.cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
 	  break;
+	case CS2_OPTION_SAMPLES_PER_SCAN:
+	  o.name = "samples-per-scan";
+	  o.title = "Samples per Scan";
+	  o.desc = "Number of samples per scan";
+	  o.type = SANE_TYPE_INT;
+	  o.unit = SANE_UNIT_NONE;
+	  o.size = WSIZE;
+	  o.cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
+	  if (s->type != CS2_TYPE_LS2000 && s->type != CS2_TYPE_LS4000
+		  && s->type != CS2_TYPE_LS5000 && s->type != CS2_TYPE_LS8000)
+		o.cap |= SANE_CAP_INACTIVE;
+	  o.constraint_type = SANE_CONSTRAINT_RANGE;
+	  range = (SANE_Range *) cs2_xmalloc (sizeof (SANE_Range));
+	  if (! range)
+		alloc_failed = 1;
+	  else
+		{
+		  range->min = 1;
+		  range->max = 16;
+		  range->quant = 1;
+		  o.constraint.range = range;
+		}
+	  break;
 	case CS2_OPTION_DEPTH:
 	  o.name = "depth";
 	  o.title = "Bit depth per channel";
@@ -994,6 +1019,7 @@ sane_open (SANE_String_Const name, SANE_Handle * h)
   s->negative = SANE_FALSE;
   s->depth = 8;
   s->infrared = 0;
+  s->samples_per_scan = 1;
   s->i_frame = 1;
   s->subframe = 0.;
   s->res = s->resx = s->resx_max;
@@ -1080,6 +1106,9 @@ sane_control_option (SANE_Handle h, SANE_Int n, SANE_Action a, void *v,
 	case CS2_OPTION_INFRARED:
 	  *(SANE_Word *) v = s->infrared;
 	  break;
+	case CS2_OPTION_SAMPLES_PER_SCAN:
+	  *(SANE_Word *) v = s->samples_per_scan;
+	  break;
 	case CS2_OPTION_DEPTH:
 	  *(SANE_Word *) v = s->depth;
 	  break;
@@ -1231,6 +1260,9 @@ sane_control_option (SANE_Handle h, SANE_Int n, SANE_Action a, void *v,
 	  s->infrared = *(SANE_Word *) v;
 	  /*      flags |= SANE_INFO_RELOAD_PARAMS; XXXXXXXXXXXXXXXXX */
 	  break;
+	case CS2_OPTION_SAMPLES_PER_SCAN:
+	  s->samples_per_scan = *(SANE_Word *) v;
+	  break;
 	case CS2_OPTION_DEPTH:
 	  s->depth = *(SANE_Word *) v;
 	  flags |= SANE_INFO_RELOAD_PARAMS;
@@ -1462,9 +1494,10 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
   SANE_Status status;
   ssize_t xfer_len_in, xfer_len_line, xfer_len_out;
   unsigned long index;
-  int colour;
+  int colour, n_colours, sample_pass;
   uint8_t *s8 = NULL;
   uint16_t *s16 = NULL;
+  double m_avg_sum;
   SANE_Byte *line_buf_new;
 
   DBG (10, "sane_read() called, maxlen = %i.\n", maxlen);
@@ -1554,6 +1587,9 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
       s->n_line_buf = xfer_len_line;
     }
 
+  /* adapt for multi-sampling */
+  xfer_len_in *= s->samples_per_scan;
+
   cs2_scanner_ready (s, CS2_STATUS_READY);
   cs2_init_buffer (s);
   cs2_parse_cmd (s, "28 00 00 00 00 00");
@@ -1570,24 +1606,44 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
       return status;
     }
 
+  n_colours = s->n_colour_out +
+	  (s->infrared_stage == CS2_INFRARED_IN ? 1 : 0);
+
   for (index = 0; index < s->logical_width; index++)
-    for (colour = 0;
-	 colour < s->n_colour_out + (s->infrared_stage ==
-				     CS2_INFRARED_IN ? 1 : 0); colour++)
+    for (colour = 0; colour < n_colours; colour++) {
+	  m_avg_sum = 0.0;
       switch (s->bytes_per_pixel)
 	{
 	case 1:
+	  /* calculate target address */
 	  if ((s->infrared_stage == CS2_INFRARED_IN)
 	      && (colour == s->n_colour_out))
 	    s8 = (uint8_t *) & (s->infrared_buf[s->infrared_index++]);
 	  else
 	    s8 =
 	      (uint8_t *) & (s->line_buf[s->n_colour_out * index + colour]);
-	  *s8 =
-	    s->recv_buf[colour * s->logical_width +
-			(colour + 1) * s->odd_padding + index];
+
+	  if (s->samples_per_scan > 1)
+		{
+		  /* calculate average of multi samples */
+		  for (sample_pass = 0;
+			   sample_pass < s->samples_per_scan;
+			   sample_pass++)
+			m_avg_sum += (double)
+			  s->recv_buf[s->logical_width *
+			  (sample_pass * n_colours + colour) +
+			  (colour + 1) * s->odd_padding + index];
+
+		  *s8 = (uint8_t) (m_avg_sum / s->samples_per_scan + 0.5);
+		}
+	  else
+		/* shortcut for single sample */
+		*s8 =
+		  s->recv_buf[colour * s->logical_width +
+					  (colour + 1) * s->odd_padding + index];
 	  break;
 	case 2:
+	  /* calculate target address */
 	  if ((s->infrared_stage == CS2_INFRARED_IN)
 	      && (colour == s->n_colour_out))
 	    s16 =
@@ -1597,9 +1653,24 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
 	      (uint16_t *) & (s->
 			       line_buf[2 *
 					(s->n_colour_out * index + colour)]);
-	  *s16 =
-	    s->recv_buf[2 * (colour * s->logical_width + index)] * 256 +
-	    s->recv_buf[2 * (colour * s->logical_width + index) + 1];
+
+	  if (s->samples_per_scan > 1)
+		{
+		  /* calculate average of multi samples */
+		  for (sample_pass = 0;
+			   s->samples_per_scan > 1 && sample_pass < s->samples_per_scan;
+			   sample_pass++)
+			m_avg_sum += (double)
+			  (s->recv_buf[2 * (s->logical_width * (sample_pass * n_colours + colour) + index)] * 256 +
+			   s->recv_buf[2 * (s->logical_width * (sample_pass * n_colours + colour) + index) + 1]);
+
+		  *s16 = (uint16_t) (m_avg_sum / s->samples_per_scan + 0.5);
+		}
+	  else
+		/* shortcut for single sample */
+		*s16 =
+		  s->recv_buf[2 * (colour * s->logical_width + index)] * 256 +
+		  s->recv_buf[2 * (colour * s->logical_width + index) + 1];
 	  *s16 <<= s->shift_bits;
 	  break;
 	default:
@@ -1608,6 +1679,7 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
 	  return SANE_STATUS_INVAL;
 	  break;
 	}
+	}
   s->xfer_position += xfer_len_line;
 
   xfer_len_out = xfer_len_line;
@@ -1683,7 +1755,8 @@ cs2_open (const char *device, cs2_interface_t interface, cs2_t ** sp)
 {
   SANE_Status status;
   cs2_t *s;
-  char *prefix = NULL, *line, *device2;
+  char *prefix = NULL, *line;
+  const char *device2;
   int i;
   int alloc_failed = 0;
   SANE_Device **device_list_new;
@@ -2870,7 +2943,7 @@ cs2_scan (cs2_t * s, cs2_scan_t type)
       cs2_pack_byte (s, 0x05);	/* image composition CCCCCCC */
       cs2_pack_byte (s, s->real_depth);	/* pixel composition */
       cs2_parse_cmd (s, "00 00 00 00 00 00 00 00 00 00 00 00 00");
-      cs2_pack_byte (s, 0x00);	/* multiread, ordering */
+      cs2_pack_byte (s, ((s->samples_per_scan - 1) << 4) + 0x00);	/* multiread, ordering */
       /* No need to use an undocumented bit in LS50 */
       if ((s->type == CS2_TYPE_LS50) || (s->type == CS2_TYPE_LS5000))
         cs2_pack_byte (s, 0x00 + (s->negative ? 0 : 1));	/* averaging, pos/neg */
@@ -2892,7 +2965,10 @@ cs2_scan (cs2_t * s, cs2_scan_t type)
 	  DBG (1, "BUG: cs2_scan(): Unknown scanning type.\n");
 	  return SANE_STATUS_INVAL;
 	}
-      cs2_pack_byte (s, 0x02);	/* scanning mode */
+      if (s->samples_per_scan == 1)
+        cs2_pack_byte (s, 0x02);	/* scanning mode single */
+      else
+        cs2_pack_byte (s, 0x10);	/* scanning mode multi */
       cs2_pack_byte (s, 0x02);	/* colour interleaving */
       cs2_pack_byte (s, 0xff);	/* (ae) */
       if (i_colour == 3)	/* infrared */
