@@ -4,44 +4,44 @@
    Copyright (C) 2005-2013 Stephane Voltz <stef.dev@free.fr>
    Copyright (C) 2006 Laurent Charpentier <laurent_pubs@yahoo.com>
    Copyright (C) 2009 Pierre Willenbrock <pierre@pirsoft.dnsalias.org>
-   
+
    This file is part of the SANE package.
-   
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA.
-   
+
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
-   
+
    The exception is that, if you link a SANE library with other files
    to produce an executable, this does not by itself cause the
    resulting executable to be covered by the GNU General Public
    License.  Your use of that executable is in no way restricted on
    account of linking the SANE library code into it.
-   
+
    This exception does not, however, invalidate any other reasons why
    the executable file might be covered by the GNU General Public
    License.
-   
+
    If you submit changes to SANE to the maintainers to be included in
    a subsequent release, you agree by submitting the changes that
    those changes may be distributed with this exception intact.
-   
+
    If you write modifications of your own for SANE, it is your choice
    whether to permit this exception to apply to your modifications.
-   If you do not wish that, delete this exception notice. 
+   If you do not wish that, delete this exception notice.
 */
 
 #ifndef GENESYS_H
@@ -49,6 +49,16 @@
 
 #include "genesys_low.h"
 
+#ifndef PATH_MAX
+# define PATH_MAX	1024
+#endif
+
+#if defined(_WIN32) || defined(HAVE_OS2_H)
+# define PATH_SEP	'\\'
+#else
+# define PATH_SEP	'/'
+#endif
+
 
 #define ENABLE(OPTION)  s->opt[OPTION].cap &= ~SANE_CAP_INACTIVE
 #define DISABLE(OPTION) s->opt[OPTION].cap |=  SANE_CAP_INACTIVE
@@ -110,6 +120,7 @@ enum Genesys_Option
   OPT_DISABLE_INTERPOLATION,
   OPT_COLOR_FILTER,
   OPT_CALIBRATION_FILE,
+  OPT_EXPIRATION_TIME,
 
   OPT_SENSOR_GROUP,
   OPT_SCAN_SW,
@@ -146,4 +157,9 @@ typedef struct Genesys_Scanner
   SANE_Int bpp_list[5];			   /**< */
 } Genesys_Scanner;
 
+#ifdef UNIT_TESTING
+SANE_Status genesys_dark_white_shading_calibration (Genesys_Device * dev);
+char *calibration_filename(Genesys_Device *currdev);
+void add_device(Genesys_Device *dev);
+#endif
 #endif /* not GENESYS_H */
