@@ -1,45 +1,45 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2012-2013 Stphane Voltz <stef.dev@free.fr>
-   
-    
+   Copyright (C) 2012-2013 Stéphane Voltz <stef.dev@free.fr>
+
+
    This file is part of the SANE package.
-   
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA.
-   
+
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
-   
+
    The exception is that, if you link a SANE library with other files
    to produce an executable, this does not by itself cause the
    resulting executable to be covered by the GNU General Public
    License.  Your use of that executable is in no way restricted on
    account of linking the SANE library code into it.
-   
+
    This exception does not, however, invalidate any other reasons why
    the executable file might be covered by the GNU General Public
    License.
-   
+
    If you submit changes to SANE to the maintainers to be included in
    a subsequent release, you agree by submitting the changes that
    those changes may be distributed with this exception intact.
-   
+
    If you write modifications of your own for SANE, it is your choice
    whether to permit this exception to apply to your modifications.
-   If you do not wish that, delete this exception notice. 
+   If you do not wish that, delete this exception notice.
 */
 
 /** @file
@@ -62,7 +62,7 @@
 
 /** @brief read scanned data
  * Read in 0xeff0 maximum sized blocks. This read is done in 2
- * parts if not multple of 512. First read is rounded to a multiple of 512 bytes, last read fetches the 
+ * parts if not multple of 512. First read is rounded to a multiple of 512 bytes, last read fetches the
  * remainder. Read addr is always 0x10000000 with the memory layout setup.
  * @param dev device to read data from
  * @param addr address within ASIC memory space, unused but kept for API
@@ -117,7 +117,7 @@ gl846_bulk_read_data (Genesys_Device * dev, uint8_t addr,
       if (status != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_error, "%s failed while writing command: %s\n",
-	       __FUNCTION__, sane_strstatus (status));
+	       __func__, sane_strstatus (status));
 	  return status;
 	}
 
@@ -128,7 +128,7 @@ gl846_bulk_read_data (Genesys_Device * dev, uint8_t addr,
 	  read /= 512;
 	  read *= 512;
 	}
-     
+
       DBG (DBG_io2,
 	   "gl846_bulk_read_data: trying to read %lu bytes of data\n",
 	   (u_long) read);
@@ -162,7 +162,7 @@ gl846_bulk_read_data (Genesys_Device * dev, uint8_t addr,
           DBG (DBG_io2, "gl846_bulk_read_data: %lu bytes of data read\n", (u_long) done);
 	}
 
-      DBG (DBG_io2, "%s: read %lu bytes, %lu remaining\n", __FUNCTION__,
+      DBG (DBG_io2, "%s: read %lu bytes, %lu remaining\n", __func__,
 	   (u_long) size, (u_long) (target - size));
 
       target -= size;
@@ -180,7 +180,7 @@ gl846_bulk_read_data (Genesys_Device * dev, uint8_t addr,
 }
 
 /****************************************************************************
- Mid level functions 
+ Mid level functions
  ****************************************************************************/
 
 static SANE_Bool
@@ -269,7 +269,7 @@ gl846_get_step_multiplier (Genesys_Register_Set * regs)
       value = (r->value & 0x0f)>>1;
       value = 1 << value;
     }
-  DBG (DBG_io, "%s: step multiplier is %d\n", __FUNCTION__, value);
+  DBG (DBG_io, "%s: step multiplier is %d\n", __func__, value);
   return value;
 }
 
@@ -304,7 +304,7 @@ static Sensor_Profile *get_sensor_profile(int sensor_type, int dpi)
             }
           else
             {
-              if(sensors[i].dpi>=dpi 
+              if(sensors[i].dpi>=dpi
               && sensors[i].dpi<sensors[idx].dpi)
                 {
                   idx=i;
@@ -317,7 +317,7 @@ static Sensor_Profile *get_sensor_profile(int sensor_type, int dpi)
   /* default fallback */
   if(idx<0)
     {
-      DBG (DBG_warn,"%s: using default sensor profile\n",__FUNCTION__);
+      DBG (DBG_warn,"%s: using default sensor profile\n",__func__);
       idx=0;
     }
 
@@ -560,7 +560,7 @@ gl846_init_registers (Genesys_Device * dev)
   DBGCOMPLETED;
 }
 
-/**@brief send slope table for motor movement 
+/**@brief send slope table for motor movement
  * Send slope_table in machine byte order
  * @param dev device to send slope table
  * @param table_nr index of the slope table in ASIC memory
@@ -577,13 +577,13 @@ gl846_send_slope_table (Genesys_Device * dev, int table_nr,
   int i;
   char msg[10000];
 
-  DBG (DBG_proc, "%s (table_nr = %d, steps = %d)\n", __FUNCTION__,
+  DBG (DBG_proc, "%s (table_nr = %d, steps = %d)\n", __func__,
        table_nr, steps);
 
   /* sanity check */
   if(table_nr<0 || table_nr>4)
     {
-      DBG (DBG_error, "%s: invalid table number %d!\n", __FUNCTION__, table_nr);
+      DBG (DBG_error, "%s: invalid table number %d!\n", __func__, table_nr);
       return SANE_STATUS_INVAL;
     }
 
@@ -601,7 +601,7 @@ gl846_send_slope_table (Genesys_Device * dev, int table_nr,
 	{
 	  sprintf (msg+strlen(msg), "%d", slope_table[i]);
 	}
-      DBG (DBG_io, "%s: %s\n", __FUNCTION__, msg);
+      DBG (DBG_io, "%s: %s\n", __func__, msg);
     }
 
   /* slope table addresses are fixed */
@@ -610,7 +610,7 @@ gl846_send_slope_table (Genesys_Device * dev, int table_nr,
     {
       DBG (DBG_error,
 	   "%s: write to AHB failed writing slope table %d (%s)\n",
-	   __FUNCTION__, table_nr, sane_strstatus (status));
+	   __func__, table_nr, sane_strstatus (status));
     }
 
   free (table);
@@ -633,7 +633,7 @@ gl846_set_adi_fe (Genesys_Device * dev, uint8_t set)
 
   /* wait for FE to be ready */
   status = sanei_genesys_get_status (dev, &val8);
-  while (val8 & REG41_FEBUSY);
+  while (val8 & REG41_FEBUSY)
     {
       usleep (10000);
       status = sanei_genesys_get_status (dev, &val8);
@@ -641,7 +641,7 @@ gl846_set_adi_fe (Genesys_Device * dev, uint8_t set)
 
   if (set == AFE_INIT)
     {
-      DBG (DBG_proc, "%s(): setting DAC %u\n", __FUNCTION__, dev->model->dac_type);
+      DBG (DBG_proc, "%s(): setting DAC %u\n", __func__, dev->model->dac_type);
 
       /* sets to default values */
       sanei_genesys_init_fe (dev);
@@ -652,7 +652,7 @@ gl846_set_adi_fe (Genesys_Device * dev, uint8_t set)
   status = sanei_genesys_fe_write_data (dev, 0x00, val);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (DBG_error, "%s: failed to write reg0: %s\n", __FUNCTION__,
+      DBG (DBG_error, "%s: failed to write reg0: %s\n", __func__,
 	   sane_strstatus (status));
       return status;
     }
@@ -660,7 +660,7 @@ gl846_set_adi_fe (Genesys_Device * dev, uint8_t set)
   status = sanei_genesys_fe_write_data (dev, 0x01, val);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (DBG_error, "%s: failed to write reg1: %s\n", __FUNCTION__,
+      DBG (DBG_error, "%s: failed to write reg1: %s\n", __func__,
 	   sane_strstatus (status));
       return status;
     }
@@ -672,7 +672,7 @@ gl846_set_adi_fe (Genesys_Device * dev, uint8_t set)
       if (status != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_error,
-	       "%s: failed to write gain %d: %s\n", __FUNCTION__, i,
+	       "%s: failed to write gain %d: %s\n", __func__, i,
 	       sane_strstatus (status));
 	  return status;
 	}
@@ -684,7 +684,7 @@ gl846_set_adi_fe (Genesys_Device * dev, uint8_t set)
       if (status != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_error,
-	       "%s: failed to write offset %d: %s\n", __FUNCTION__, i,
+	       "%s: failed to write offset %d: %s\n", __func__, i,
 	       sane_strstatus (status));
 	  return status;
 	}
@@ -776,15 +776,15 @@ gl846_init_motor_regs_scan (Genesys_Device * dev,
 
   use_fast_fed=0;
   /* no fast fed since feed works well */
-  if(dev->settings.yres==4444 && feed_steps>100 
+  if(dev->settings.yres==4444 && feed_steps>100
      && ((flags & MOTOR_FLAG_FEED)==0))
     {
       use_fast_fed=1;
     }
-  DBG (DBG_io, "%s: use_fast_fed=%d\n", __FUNCTION__, use_fast_fed);
+  DBG (DBG_io, "%s: use_fast_fed=%d\n", __func__, use_fast_fed);
 
   sanei_genesys_set_triple(reg, REG_LINCNT, scan_lines);
-  DBG (DBG_io, "%s: lincnt=%d\n", __FUNCTION__, scan_lines);
+  DBG (DBG_io, "%s: lincnt=%d\n", __func__, scan_lines);
 
   /* compute register 02 value */
   r = sanei_genesys_get_address (reg, REG02);
@@ -845,7 +845,7 @@ gl846_init_motor_regs_scan (Genesys_Device * dev,
 
   /* correct move distance by acceleration and deceleration amounts */
   feedl=feed_steps;
-  if (use_fast_fed) 
+  if (use_fast_fed)
     {
         feedl<<=fast_step_type;
         dist=(scan_steps+2*fast_steps)*factor;
@@ -863,8 +863,8 @@ gl846_init_motor_regs_scan (Genesys_Device * dev,
       if (flags & MOTOR_FLAG_FEED)
         dist *=2;
     }
-  DBG (DBG_io2, "%s: scan steps=%d\n", __FUNCTION__, scan_steps);
-  DBG (DBG_io2, "%s: acceleration distance=%d\n", __FUNCTION__, dist);
+  DBG (DBG_io2, "%s: scan steps=%d\n", __func__, scan_steps);
+  DBG (DBG_io2, "%s: acceleration distance=%d\n", __func__, dist);
 
   /* check for overflow */
   if(dist<feedl)
@@ -873,7 +873,7 @@ gl846_init_motor_regs_scan (Genesys_Device * dev,
     feedl = 0;
 
   sanei_genesys_set_triple(reg,REG_FEEDL,feedl);
-  DBG (DBG_io ,"%s: feedl=%d\n",__FUNCTION__,feedl);
+  DBG (DBG_io ,"%s: feedl=%d\n",__func__,feedl);
 
   r = sanei_genesys_get_address (reg, REG0C);
   ccdlmt=(r->value & REG0C_CCDLMT)+1;
@@ -887,7 +887,7 @@ gl846_init_motor_regs_scan (Genesys_Device * dev,
   */
 
   /* if quarter step, bipolar Vref2 */
-  /* XXX STEF XXX GPIO 
+  /* XXX STEF XXX GPIO
   if (scan_step_type > 1)
     {
       if (scan_step_type < 3)
@@ -1021,16 +1021,16 @@ gl846_init_optical_regs_scan (Genesys_Device * dev,
        "half_ccd=%d, flags=%x\n", exposure_time,
        used_res, start, pixels, channels, depth, half_ccd, flags);
 
-  /* resolution is divided according to CKSEL */ 
+  /* resolution is divided according to CKSEL */
   r = sanei_genesys_get_address (reg, REG18);
   cksel= (r->value & REG18_CKSEL)+1;
-  DBG (DBG_io2, "%s: cksel=%d\n", __FUNCTION__, cksel);
-  
+  DBG (DBG_io2, "%s: cksel=%d\n", __func__, cksel);
+
   /* to manage high resolution device while keeping good
    * low resolution scanning speed, we make hardware dpi vary */
   dpihw=sanei_genesys_compute_dpihw(dev, used_res * cksel);
   factor=dev->sensor.optical_res/dpihw;
-  DBG (DBG_io2, "%s: dpihw=%d (factor=%d)\n", __FUNCTION__, dpihw, factor);
+  DBG (DBG_io2, "%s: dpihw=%d (factor=%d)\n", __func__, dpihw, factor);
 
   /* sensor parameters */
   sensor=get_sensor_profile(dev->model->ccd_type, dpihw);
@@ -1040,7 +1040,7 @@ gl846_init_optical_regs_scan (Genesys_Device * dev,
   /* start and end coordinate in optical dpi coordinates */
   startx = start/cksel+dev->sensor.CCD_start_xoffset;
   used_pixels=pixels/cksel;
- 
+
   /* end of sensor window */
   endx = startx + used_pixels;
 
@@ -1184,7 +1184,7 @@ gl846_init_optical_regs_scan (Genesys_Device * dev,
 	{
 	  r->value |= REG87_LEDADD;
 	}
-      /* RGB weighting 
+      /* RGB weighting
       r = sanei_genesys_get_address (reg, 0x01);
       r->value &= ~REG01_TRUEGRAY;
       if (channels == 1 && (flags & OPTICAL_FLAG_ENABLE_LEDADD))
@@ -1215,21 +1215,21 @@ gl846_init_optical_regs_scan (Genesys_Device * dev,
   dev->line_interp = 0;
 
   sanei_genesys_set_double(reg,REG_DPISET,dpiset);
-  DBG (DBG_io2, "%s: dpiset used=%d\n", __FUNCTION__, dpiset);
+  DBG (DBG_io2, "%s: dpiset used=%d\n", __func__, dpiset);
 
   sanei_genesys_set_double(reg,REG_STRPIXEL,startx);
   sanei_genesys_set_double(reg,REG_ENDPIXEL,endx);
-  DBG (DBG_io2, "%s: startx=%d\n", __FUNCTION__, startx);
-  DBG (DBG_io2, "%s: endx  =%d\n", __FUNCTION__, endx);
+  DBG (DBG_io2, "%s: startx=%d\n", __func__, startx);
+  DBG (DBG_io2, "%s: endx  =%d\n", __func__, endx);
+
+  DBG (DBG_io2, "%s: used_pixels=%d\n", __func__, used_pixels);
+  DBG (DBG_io2, "%s: pixels     =%d\n", __func__, pixels);
+  DBG (DBG_io2, "%s: depth      =%d\n", __func__, depth);
+  DBG (DBG_io2, "%s: dev->bpl   =%lu\n", __func__, (unsigned long)dev->bpl);
+  DBG (DBG_io2, "%s: dev->len   =%lu\n", __func__, (unsigned long)dev->len);
+  DBG (DBG_io2, "%s: dev->dist  =%lu\n", __func__, (unsigned long)dev->dist);
+  DBG (DBG_io2, "%s: dev->segnb =%lu\n", __func__, (unsigned long)dev->segnb);
 
-  DBG (DBG_io2, "%s: used_pixels=%d\n", __FUNCTION__, used_pixels);
-  DBG (DBG_io2, "%s: pixels     =%d\n", __FUNCTION__, pixels);
-  DBG (DBG_io2, "%s: depth      =%d\n", __FUNCTION__, depth);
-  DBG (DBG_io2, "%s: dev->bpl   =%lu\n", __FUNCTION__, (unsigned long)dev->bpl);
-  DBG (DBG_io2, "%s: dev->len   =%lu\n", __FUNCTION__, (unsigned long)dev->len);
-  DBG (DBG_io2, "%s: dev->dist  =%lu\n", __FUNCTION__, (unsigned long)dev->dist);
-  DBG (DBG_io2, "%s: dev->segnb =%lu\n", __FUNCTION__, (unsigned long)dev->segnb);
-  
   words_per_line *= channels;
   dev->wpl = words_per_line;
 
@@ -1241,14 +1241,14 @@ gl846_init_optical_regs_scan (Genesys_Device * dev,
 
   /* MAXWD is expressed in 4 words unit */
   sanei_genesys_set_triple(reg, REG_MAXWD, (words_per_line >> 2));
-  DBG (DBG_io2, "%s: words_per_line used=%d\n", __FUNCTION__, words_per_line);
+  DBG (DBG_io2, "%s: words_per_line used=%d\n", __func__, words_per_line);
 
   sanei_genesys_set_double(reg, REG_LPERIOD, exposure_time);
-  DBG (DBG_io2, "%s: exposure_time used=%d\n", __FUNCTION__, exposure_time);
+  DBG (DBG_io2, "%s: exposure_time used=%d\n", __func__, exposure_time);
 
   r = sanei_genesys_get_address (reg, 0x34);
   r->value = dev->sensor.dummy_pixel;
-  
+
   DBGCOMPLETED;
   return SANE_STATUS_GOOD;
 }
@@ -1383,7 +1383,7 @@ gl846_init_scan_regs (Genesys_Device * dev,
       depth = 8;
     }
 
-  /* we enable true gray for cis scanners only, and just when doing 
+  /* we enable true gray for cis scanners only, and just when doing
    * scan since color calibration is OK for this mode
    */
   oflags = 0;
@@ -1393,7 +1393,7 @@ gl846_init_scan_regs (Genesys_Device * dev,
     oflags |= OPTICAL_FLAG_DISABLE_GAMMA;
   if(flags & SCAN_FLAG_DISABLE_LAMP)
     oflags |= OPTICAL_FLAG_DISABLE_LAMP;
-  
+
   if (dev->model->is_cis && dev->settings.true_gray)
     {
       oflags |= OPTICAL_FLAG_ENABLE_LEDADD;
@@ -1504,12 +1504,12 @@ gl846_init_scan_regs (Genesys_Device * dev,
   /* theory :
      target_size =
      (dev->settings.pixels * dev->settings.lines * channels * depth) / 8;
-     but it suffers from integer overflow so we do the following: 
+     but it suffers from integer overflow so we do the following:
 
-     1 bit color images store color data byte-wise, eg byte 0 contains 
-     8 bits of red data, byte 1 contains 8 bits of green, byte 2 contains 
+     1 bit color images store color data byte-wise, eg byte 0 contains
+     8 bits of red data, byte 1 contains 8 bits of green, byte 2 contains
      8 bits of blue.
-     This does not fix the overflow, though. 
+     This does not fix the overflow, though.
      644mp*16 = 10gp, leading to an overflow
      -- pierre
    */
@@ -1645,7 +1645,7 @@ gl846_calculate_current_setup (Genesys_Device * dev)
   slope_dpi = slope_dpi * (1 + dummy);
 
   exposure_time = gl846_compute_exposure (dev, used_res);
-  DBG (DBG_info, "%s : exposure_time=%d pixels\n", __FUNCTION__, exposure_time);
+  DBG (DBG_info, "%s : exposure_time=%d pixels\n", __func__, exposure_time);
 
   /* max_shift */
   max_shift=sanei_genesys_compute_max_shift(dev,channels,yres,0);
@@ -1761,7 +1761,7 @@ gl846_stop_action (Genesys_Device * dev)
   if (status != SANE_STATUS_GOOD)
     {
       DBG (DBG_error,
-	   "%s: failed to read home sensor: %s\n", __FUNCTION__,
+	   "%s: failed to read home sensor: %s\n", __func__,
 	   sane_strstatus (status));
       DBGCOMPLETED;
       return status;
@@ -1770,7 +1770,7 @@ gl846_stop_action (Genesys_Device * dev)
   /* only stop action if needed */
   if (!(val40 & REG40_DATAENB) && !(val40 & REG40_MOTMFLG))
     {
-      DBG (DBG_info, "%s: already stopped\n", __FUNCTION__);
+      DBG (DBG_info, "%s: already stopped\n", __func__);
       DBGCOMPLETED;
       return SANE_STATUS_GOOD;
     }
@@ -1783,7 +1783,7 @@ gl846_stop_action (Genesys_Device * dev)
   if (status != SANE_STATUS_GOOD)
     {
       DBG (DBG_error,
-	   "%s: failed to write register 01: %s\n", __FUNCTION__,
+	   "%s: failed to write register 01: %s\n", __func__,
 	   sane_strstatus (status));
       return status;
     }
@@ -1801,7 +1801,7 @@ gl846_stop_action (Genesys_Device * dev)
       if (status != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_error,
-	       "%s: failed to read home sensor: %s\n", __FUNCTION__,
+	       "%s: failed to read home sensor: %s\n", __func__,
 	       sane_strstatus (status));
           DBGCOMPLETED;
 	  return status;
@@ -1962,20 +1962,20 @@ gl846_slow_back_home (Genesys_Device * dev, SANE_Bool wait_until_home)
   /* is sensor at home? */
   if (val & HOMESNR)
     {
-      DBG (DBG_info, "%s: already at home, completed\n", __FUNCTION__);
+      DBG (DBG_info, "%s: already at home, completed\n", __func__);
       dev->scanhead_position_in_steps = 0;
       DBGCOMPLETED;
       return SANE_STATUS_GOOD;
     }
 
   memcpy (local_reg, dev->reg, GENESYS_GL846_MAX_REGS * sizeof (Genesys_Register_Set));
-  
+
   resolution=sanei_genesys_get_lowest_ydpi(dev);
-  
+
   /* TODO add scan_mode to the API */
   scan_mode= dev->settings.scan_mode;
   dev->settings.scan_mode=SCAN_MODE_LINEART;
-  gl846_init_scan_regs (dev,
+  status = gl846_init_scan_regs (dev,
 			local_reg,
 			resolution,
 			resolution,
@@ -1989,11 +1989,19 @@ gl846_slow_back_home (Genesys_Device * dev, SANE_Bool wait_until_home)
 			SCAN_FLAG_DISABLE_SHADING |
 			SCAN_FLAG_DISABLE_GAMMA |
 			SCAN_FLAG_IGNORE_LINE_DISTANCE);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_error,
+           "gl846_slow_back_home: failed to set up registers: %s\n",
+           sane_strstatus (status));
+      DBGCOMPLETED;
+      return status;
+    }
   dev->settings.scan_mode=scan_mode;
 
   /* clear scan and feed count */
   RIE (sanei_genesys_write_register (dev, REG0D, REG0D_CLRLNCNT | REG0D_CLRMCNT));
-  
+
   /* set up for reverse */
   r = sanei_genesys_get_address (local_reg, REG02);
   r->value |= REG02_MTRREV;
@@ -2153,7 +2161,7 @@ gl846_search_start_position (Genesys_Device * dev)
   memcpy (dev->reg, local_reg,
 	  GENESYS_GL846_MAX_REGS * sizeof (Genesys_Register_Set));
 
-/*TODO: find out where sanei_genesys_search_reference_point 
+/*TODO: find out where sanei_genesys_search_reference_point
   stores information, and use that correctly*/
   status =
     sanei_genesys_search_reference_point (dev, data, 0, dpi, pixels,
@@ -2171,7 +2179,7 @@ gl846_search_start_position (Genesys_Device * dev)
   return SANE_STATUS_GOOD;
 }
 
-/* 
+/*
  * sets up register for coarse gain calibration
  * todo: check it for scanners using it */
 static SANE_Status
@@ -2249,13 +2257,13 @@ gl846_feed (Genesys_Device * dev, unsigned int steps)
   uint8_t val;
 
   DBGSTART;
-  DBG (DBG_io, "%s: steps=%d\n", __FUNCTION__, steps);
+  DBG (DBG_io, "%s: steps=%d\n", __func__, steps);
 
   /* prepare local registers */
   memcpy (local_reg, dev->reg, GENESYS_GL846_MAX_REGS * sizeof (Genesys_Register_Set));
 
   resolution=sanei_genesys_get_lowest_ydpi(dev);
-  gl846_init_scan_regs (dev,
+  status = gl846_init_scan_regs (dev,
 			local_reg,
 			resolution,
 			resolution,
@@ -2270,6 +2278,14 @@ gl846_feed (Genesys_Device * dev, unsigned int steps)
 			SCAN_FLAG_DISABLE_GAMMA |
                         SCAN_FLAG_FEEDING |
 			SCAN_FLAG_IGNORE_LINE_DISTANCE);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_error,
+           "gl846_feed: failed to set up registers: %s\n",
+           sane_strstatus (status));
+      DBGCOMPLETED;
+      return status;
+    }
 
   /* set exposure to zero */
   sanei_genesys_set_triple(local_reg,REG_EXPR,0);
@@ -2279,18 +2295,18 @@ gl846_feed (Genesys_Device * dev, unsigned int steps)
   /* clear scan and feed count */
   RIE (sanei_genesys_write_register (dev, REG0D, REG0D_CLRLNCNT));
   RIE (sanei_genesys_write_register (dev, REG0D, REG0D_CLRMCNT));
-  
+
   /* set up for no scan */
   r = sanei_genesys_get_address (local_reg, REG01);
   r->value &= ~REG01_SCAN;
-  
+
   /* send registers */
   RIE (dev->model->cmd_set->bulk_write_register (dev, local_reg, GENESYS_GL846_MAX_REGS));
 
   status = gl846_start_action (dev);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (DBG_error, "%s: failed to start motor: %s\n", __FUNCTION__, sane_strstatus (status));
+      DBG (DBG_error, "%s: failed to start motor: %s\n", __func__, sane_strstatus (status));
       gl846_stop_action (dev);
 
       /* restore original registers */
@@ -2308,7 +2324,7 @@ gl846_feed (Genesys_Device * dev, unsigned int steps)
 
   /* then stop scanning */
   RIE(gl846_stop_action (dev));
-  
+
   DBGCOMPLETED;
   return SANE_STATUS_GOOD;
 }
@@ -2332,10 +2348,10 @@ gl846_init_regs_for_shading (Genesys_Device * dev)
   if(dev->calib_resolution==4800)
     dev->calib_lines *= 2;
   dev->calib_pixels = (dev->sensor.sensor_pixels*dev->calib_resolution)/dev->sensor.optical_res;
-  DBG (DBG_io, "%s: calib_lines  = %d\n", __FUNCTION__, (unsigned int)dev->calib_lines);
-  DBG (DBG_io, "%s: calib_pixels = %d\n", __FUNCTION__, (unsigned int)dev->calib_pixels);
+  DBG (DBG_io, "%s: calib_lines  = %d\n", __func__, (unsigned int)dev->calib_lines);
+  DBG (DBG_io, "%s: calib_pixels = %d\n", __func__, (unsigned int)dev->calib_pixels);
 
-  /* this is aworkaround insufficent distance for slope 
+  /* this is aworkaround insufficent distance for slope
    * motor acceleration TODO special motor slope for shading  */
   move=1;
   if(dev->calib_resolution<1200)
@@ -2361,14 +2377,14 @@ gl846_init_regs_for_shading (Genesys_Device * dev)
 
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (DBG_error, "%s: failed to setup scan: %s\n", __FUNCTION__, sane_strstatus (status));
+      DBG (DBG_error, "%s: failed to setup scan: %s\n", __func__, sane_strstatus (status));
       return status;
     }
- 
+
   status = dev->model->cmd_set->bulk_write_register (dev, dev->calib_reg, GENESYS_GL846_MAX_REGS);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (DBG_error, "%s: failed to bulk write registers: %s\n", __FUNCTION__, sane_strstatus (status));
+      DBG (DBG_error, "%s: failed to bulk write registers: %s\n", __func__, sane_strstatus (status));
       return status;
     }
 
@@ -2421,9 +2437,9 @@ gl846_init_regs_for_scan (Genesys_Device * dev)
 
      assumption: steps are expressed at maximum motor resolution
 
-     we need:   
-     SANE_Fixed y_offset;                       
-     SANE_Fixed y_size;                 
+     we need:
+     SANE_Fixed y_offset;             
+     SANE_Fixed y_size;       
      SANE_Fixed y_offset_calib;
      mm_to_steps()=motor dpi / 2.54 / 10=motor dpi / MM_PER_INCH */
 
@@ -2436,7 +2452,7 @@ gl846_init_regs_for_scan (Genesys_Device * dev)
   move += dev->settings.tl_y;
   move = (move * move_dpi) / MM_PER_INCH;
   move -= dev->scanhead_position_in_steps;
-  DBG (DBG_info, "%s: move=%f steps\n",__FUNCTION__, move);
+  DBG (DBG_info, "%s: move=%f steps\n",__func__, move);
 
   /* fast move to scan area */
   /* we don't move fast the whole distance since it would involve
@@ -2448,14 +2464,14 @@ gl846_init_regs_for_scan (Genesys_Device * dev)
       status = gl846_feed (dev, move-500);
       if (status != SANE_STATUS_GOOD)
         {
-          DBG (DBG_error, "%s: failed to move to scan area\n",__FUNCTION__);
+          DBG (DBG_error, "%s: failed to move to scan area\n",__func__);
           return status;
         }
       move=500;
     }
- 
+
   DBG (DBG_info, "gl846_init_regs_for_scan: move=%f steps\n", move);
-  DBG (DBG_info, "%s: move=%f steps\n", __FUNCTION__, move);
+  DBG (DBG_info, "%s: move=%f steps\n", __func__, move);
 
   /* start */
   start = SANE_UNFIX (dev->model->x_offset);
@@ -2465,7 +2481,7 @@ gl846_init_regs_for_scan (Genesys_Device * dev)
   flags = 0;
 
   /* emulated lineart from gray data is required for now */
-  if(dev->settings.scan_mode == SCAN_MODE_LINEART 
+  if(dev->settings.scan_mode == SCAN_MODE_LINEART
      && dev->settings.dynamic_lineart)
     {
       flags |= SCAN_FLAG_DYNAMIC_LINEART;
@@ -2489,7 +2505,7 @@ gl846_init_regs_for_scan (Genesys_Device * dev)
 
   if (status != SANE_STATUS_GOOD)
     return status;
-  
+
   DBGCOMPLETED;
   return SANE_STATUS_GOOD;
 }
@@ -2510,9 +2526,9 @@ gl846_send_shading_data (Genesys_Device * dev, uint8_t * data, int size)
   uint8_t val,*buffer,*ptr,*src;
 
   DBGSTART;
-  DBG( DBG_io2, "%s: writing %d bytes of shading data\n",__FUNCTION__,size);
+  DBG( DBG_io2, "%s: writing %d bytes of shading data\n",__func__,size);
 
-  /* shading data is plit in 3 (up to 5 with IR) areas 
+  /* shading data is plit in 3 (up to 5 with IR) areas
      write(0x10014000,0x00000dd8)
      URB 23429  bulk_out len  3544  wrote 0x33 0x10 0x....
      write(0x1003e000,0x00000dd8)
@@ -2527,10 +2543,10 @@ gl846_send_shading_data (Genesys_Device * dev, uint8_t * data, int size)
   /* compute deletion factor */
   sanei_genesys_get_double(dev->reg,REG_DPISET,&tempo);
   dpiset=tempo;
-  DBG( DBG_io2, "%s: STRPIXEL=%d, ENDPIXEL=%d, PIXELS=%d, DPISET=%d\n",__FUNCTION__,strpixel,endpixel,endpixel-strpixel,dpiset);
+  DBG( DBG_io2, "%s: STRPIXEL=%d, ENDPIXEL=%d, PIXELS=%d, DPISET=%d\n",__func__,strpixel,endpixel,endpixel-strpixel,dpiset);
   dpihw=sanei_genesys_compute_dpihw(dev,dpiset);
   factor=dpihw/dpiset;
-  DBG( DBG_io2, "%s: factor=%d\n",__FUNCTION__,factor);
+  DBG( DBG_io2, "%s: factor=%d\n",__func__,factor);
 
   if(DBG_LEVEL>=DBG_data)
     {
@@ -2542,20 +2558,20 @@ gl846_send_shading_data (Genesys_Device * dev, uint8_t * data, int size)
           fprintf(dev->binary,"P5\n%d %d\n%d\n",(endpixel-strpixel)/factor*channels,lines/channels,255);
         }
     }
-  
+
   pixels=endpixel-strpixel;
 
   /* since we're using SHDAREA, substract startx coordinate from shading */
   strpixel-=((dev->sensor.CCD_start_xoffset*600)/dev->sensor.optical_res);
-  
+
   /* turn pixel value into bytes 2x16 bits words */
-  strpixel*=2*2; 
+  strpixel*=2*2;
   pixels*=2*2;
 
   /* allocate temporary buffer */
   buffer=(uint8_t *)malloc(pixels);
   memset(buffer,0,pixels);
-  DBG( DBG_io2, "%s: using chunks of %d (0x%04x) bytes\n",__FUNCTION__,pixels,pixels);
+  DBG( DBG_io2, "%s: using chunks of %d (0x%04x) bytes\n",__func__,pixels,pixels);
 
   /* base addr of data has been written in reg D0-D4 in 4K word, so AHB address
    * is 8192*reg value */
@@ -2578,18 +2594,19 @@ gl846_send_shading_data (Genesys_Device * dev, uint8_t * data, int size)
           ptr[1]=src[1];
           ptr[2]=src[2];
           ptr[3]=src[3];
-          
+
           /* next shading coefficient */
           ptr+=4;
         }
 
-      RIE (sanei_genesys_read_register (dev, 0xd0+i, &val));
+      RIEF (sanei_genesys_read_register (dev, 0xd0+i, &val), buffer);
       addr = val * 8192 + 0x10000000;
       status = sanei_genesys_write_ahb (dev->dn, dev->usb_mode, addr, pixels, buffer);
       if (status != SANE_STATUS_GOOD)
         {
           DBG (DBG_error, "gl846_send_shading_data; write to AHB failed (%s)\n",
 	      sane_strstatus (status));
+          free(buffer);
           return status;
         }
     }
@@ -2632,7 +2649,7 @@ gl846_led_calibration (Genesys_Device * dev)
     {
       RIE(gl846_feed (dev, move));
     }
-  DBG (DBG_io, "%s: move=%f steps\n", __FUNCTION__, move);
+  DBG (DBG_io, "%s: move=%f steps\n", __func__, move);
 
   /* offset calibration is always done in color mode */
   channels = 3;
@@ -2640,7 +2657,7 @@ gl846_led_calibration (Genesys_Device * dev)
   used_res=sanei_genesys_compute_dpihw(dev,dev->settings.xres);
   sensor=get_sensor_profile(dev->model->ccd_type, used_res);
   num_pixels = (dev->sensor.sensor_pixels*used_res)/dev->sensor.optical_res;
-  
+
   /* initial calibration reg values */
   memcpy (dev->calib_reg, dev->reg, GENESYS_GL846_MAX_REGS * sizeof (Genesys_Register_Set));
 
@@ -2662,7 +2679,7 @@ gl846_led_calibration (Genesys_Device * dev)
 				 SCAN_FLAG_IGNORE_LINE_DISTANCE);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (DBG_error, "%s: failed to setup scan: %s\n", __FUNCTION__, sane_strstatus (status));
+      DBG (DBG_error, "%s: failed to setup scan: %s\n", __func__, sane_strstatus (status));
       return status;
     }
 
@@ -2770,18 +2787,18 @@ gl846_led_calibration (Genesys_Device * dev)
 
   /* cleanup before return */
   free (line);
-  
+
   /* go back home */
   if(move>20)
     {
       status=gl846_slow_back_home (dev, SANE_TRUE);
     }
- 
+
   DBGCOMPLETED;
   return status;
 }
 
-/** 
+/**
  * set up GPIO/GPOE for idle state
  */
 static SANE_Status
@@ -2799,13 +2816,13 @@ gl846_init_gpio (Genesys_Device * dev)
     }
   if(gpios[idx].sensor_id==0)
     {
-      DBG (DBG_error, "%s: failed to find GPIO profile for sensor_id=%d\n", __FUNCTION__, dev->model->ccd_type);
+      DBG (DBG_error, "%s: failed to find GPIO profile for sensor_id=%d\n", __func__, dev->model->ccd_type);
       return SANE_STATUS_INVAL;
     }
 
   RIE (sanei_genesys_write_register (dev, REGA7, gpios[idx].ra7));
   RIE (sanei_genesys_write_register (dev, REGA6, gpios[idx].ra6));
-  
+
   RIE (sanei_genesys_write_register (dev, REG6B, gpios[idx].r6b));
   RIE (sanei_genesys_write_register (dev, REG6C, gpios[idx].r6c));
   RIE (sanei_genesys_write_register (dev, REG6D, gpios[idx].r6d));
@@ -2819,7 +2836,7 @@ gl846_init_gpio (Genesys_Device * dev)
   return status;
 }
 
-/** 
+/**
  * set memory layout by filling values in dedicated registers
  */
 static SANE_Status
@@ -2840,7 +2857,7 @@ gl846_init_memory_layout (Genesys_Device * dev)
     }
   if(layouts[idx].model==NULL)
     {
-      DBG(DBG_error, "%s: failed to find memory layout for model %s!\n", __FUNCTION__, dev->model->name);
+      DBG(DBG_error, "%s: failed to find memory layout for model %s!\n", __func__, dev->model->name);
       return SANE_STATUS_INVAL;
     }
 
@@ -2867,7 +2884,7 @@ gl846_init_memory_layout (Genesys_Device * dev)
  */
 #ifndef UNIT_TESTING
 static
-#endif 
+#endif
 SANE_Status
 gl846_boot (Genesys_Device * dev, SANE_Bool cold)
 {
@@ -2882,7 +2899,7 @@ gl846_boot (Genesys_Device * dev, SANE_Bool cold)
       RIE (sanei_genesys_write_register (dev, 0x0e, 0x01));
       RIE (sanei_genesys_write_register (dev, 0x0e, 0x00));
     }
-  
+
   if(dev->usb_mode == 1)
     {
       val = 0x14;
@@ -2898,7 +2915,7 @@ gl846_boot (Genesys_Device * dev, SANE_Bool cold)
   if (val & REG40_CHKVER)
     {
       RIE (sanei_genesys_read_register (dev, 0x00, &val));
-      DBG (DBG_info, "%s: reported version for genesys chip is 0x%02x\n", __FUNCTION__, val);
+      DBG (DBG_info, "%s: reported version for genesys chip is 0x%02x\n", __func__, val);
     }
 
   /* Set default values for registers */
@@ -2950,7 +2967,7 @@ SANE_Status gl846_init (Genesys_Device * dev)
 
   DBG_INIT ();
   DBGSTART;
-  
+
   status=sanei_genesys_asic_init(dev, GENESYS_GL846_MAX_REGS);
 
   DBGCOMPLETED;
@@ -2990,7 +3007,7 @@ gl846_update_hardware_sensors (Genesys_Scanner * s)
 
 /** @brief search for a full width black or white strip.
  * This function searches for a black or white stripe across the scanning area.
- * When searching backward, the searched area must completely be of the desired 
+ * When searching backward, the searched area must completely be of the desired
  * color since this area will be used for calibration which scans forward.
  * @param dev scanner device
  * @param forward SANE_TRUE if searching forward, SANE_FALSE if searching backward
@@ -3013,7 +3030,15 @@ gl846_search_strip (Genesys_Device * dev, SANE_Bool forward, SANE_Bool black)
   DBG (DBG_proc, "gl846_search_strip %s %s\n", black ? "black" : "white",
        forward ? "forward" : "reverse");
 
-  gl846_set_fe (dev, AFE_SET);
+  status = gl846_set_fe (dev, AFE_SET);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_error,
+           "gl846_search_strip: gl846_set_fe() failed: %s\n",
+           sane_strstatus(status));
+      return status;
+    }
+
   status = gl846_stop_action (dev);
   if (status != SANE_STATUS_GOOD)
     {
@@ -3415,7 +3440,7 @@ gl846_offset_calibration (Genesys_Device * dev)
       snprintf(title,20,"offset%03d.pnm",bottom);
       sanei_genesys_write_pnm_file (title, first_line, bpp, channels, pixels, lines);
    }
-     
+
   bottomavg = dark_average (first_line, pixels, lines, channels, black_pixels);
   DBG (DBG_io2, "gl846_offset_calibration: bottom avg=%d\n", bottomavg);
 
@@ -3429,7 +3454,7 @@ gl846_offset_calibration (Genesys_Device * dev)
   DBG (DBG_info, "gl846_offset_calibration: starting second line reading\n");
   RIEF2 (gl846_begin_scan (dev, dev->calib_reg, SANE_TRUE), first_line, second_line);
   RIEF2 (sanei_genesys_read_data_from_scanner (dev, second_line, total_size), first_line, second_line);
-      
+
   topavg = dark_average (second_line, pixels, lines, channels, black_pixels);
   DBG (DBG_io2, "gl846_offset_calibration: top avg=%d\n", topavg);
 
@@ -3572,24 +3597,10 @@ gl846_coarse_gain_calibration (Genesys_Device * dev, int dpi)
       max[j] = 0;
       for (i = pixels/4; i < (pixels*3/4); i++)
 	{
-          if(bpp==16)
-            {
-	  if (dev->model->is_cis)
-	    val =
-	      line[i * 2 + j * 2 * pixels + 1] * 256 +
-	      line[i * 2 + j * 2 * pixels];
-	  else
-	    val =
-	      line[i * 2 * channels + 2 * j + 1] * 256 +
-	      line[i * 2 * channels + 2 * j];
-            }
-          else
-            {
 	  if (dev->model->is_cis)
 	    val = line[i + j * pixels];
 	  else
 	    val = line[i * channels + j];
-            }
 
 	    max[j] += val;
 	}
@@ -3619,12 +3630,6 @@ gl846_coarse_gain_calibration (Genesys_Device * dev, int dpi)
       dev->frontend.gain[2] = dev->frontend.gain[1] = dev->frontend.gain[0];
     }
 
-  if (channels == 1)
-    {
-      dev->frontend.gain[0] = dev->frontend.gain[1];
-      dev->frontend.gain[2] = dev->frontend.gain[1];
-    }
-
   free (line);
 
   RIE (gl846_stop_action (dev));
@@ -3675,6 +3680,7 @@ static Genesys_Command_Set gl846_cmd_set = {
   gl846_led_calibration,
 
   gl846_slow_back_home,
+  NULL,
 
   sanei_genesys_bulk_write_register,
   NULL,
@@ -3691,7 +3697,8 @@ static Genesys_Command_Set gl846_cmd_set = {
   NULL,
   gl846_send_shading_data,
   gl846_calculate_current_setup,
-  gl846_boot
+  gl846_boot,
+  NULL
 };
 
 SANE_Status
