@@ -54,7 +54,7 @@
    This file implements a SANE backend for the Kodak DC-240
    digital camera.  THIS IS EXTREMELY ALPHA CODE!  USE AT YOUR OWN RISK!! 
 
-   (feedback to:  peter@fales.com
+   (feedback to:  gphoto2-devel@fales-lorenz.net)
 
    This backend is based somewhat on the dc25 backend included in this
    package by Peter Fales, and the dc210 backend by Brian J. Murrell
@@ -155,12 +155,6 @@ struct cam_dirent
   long size;
 };
 
-#ifdef __GNUC__
-#define UNUSEDARG __attribute__ ((unused))
-#else
-#define UNUSEDARG
-#endif
-
 struct cam_dirlist
 {
   SANE_Char name[48];
@@ -197,4 +191,4 @@ static SANE_Int converter_fill_buffer (void);
 
 static SANE_Bool converter_scan_complete (void);
 
-static void converter_init (void);
+static SANE_Status converter_init (SANE_Handle handle);
