@@ -845,6 +845,7 @@ attach (SANE_String_Const devname, GT68xx_Device ** devp, SANE_Bool may_wait)
         {
           if (devp)
             *devp = dev;
+          dev->missing = SANE_FALSE;
           DBG (4, "attach: device `%s' was already in device list\n",
                devname);
           return SANE_STATUS_GOOD;
@@ -888,6 +889,7 @@ attach (SANE_String_Const devname, GT68xx_Device ** devp, SANE_Bool may_wait)
     }
 
   dev->file_name = strdup (devname);
+  dev->missing = SANE_FALSE;
   if (!dev->file_name)
     return SANE_STATUS_NO_MEM;
   DBG (2, "attach: found %s flatbed scanner %s at %s\n", dev->model->vendor,
@@ -1090,43 +1092,32 @@ download_firmware_file (GT68xx_Device * dev)
   return status;
 }
 
-/* -------------------------- SANE API functions ------------------------- */
-
-SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+/** probe for gt68xx devices
+ * This function scan usb and try to attached to scanner
+ * configured in gt68xx.conf .
+ */
+static SANE_Status probe_gt68xx_devices(void)
 {
   SANE_Char line[PATH_MAX];
   SANE_Char *word;
   SANE_String_Const cp;
   SANE_Int linenumber;
+  GT68xx_Device *dev;
   FILE *fp;
 
-  DBG_INIT ();
-  DBG (2, "SANE GT68xx backend version %d.%d build %d from %s\n", SANE_CURRENT_MAJOR,
-       V_MINOR, BUILD, PACKAGE_STRING);
-
-  if (version_code)
-    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
-
-  DBG (5, "sane_init: authorize %s null\n", authorize ? "!=" : "==");
-
-  sanei_usb_init ();
-
-  num_devices = 0;
-  first_dev = 0;
-  first_handle = 0;
-  devlist = 0;
+  /* set up for no new devices detected at first */
   new_dev = 0;
   new_dev_len = 0;
   new_dev_alloced = 0;
 
-#ifdef DBG_LEVEL
-  if (DBG_LEVEL > 0)
+  /* mark already detected devices as missing, during device probe
+   * detected devices will clear this flag */
+  dev = first_dev;
+  while(dev!=NULL)
     {
-      DBG (5, "sane_init: debug options are enabled, handle with care\n");
-      debug_options = SANE_TRUE;
+      dev->missing = SANE_TRUE;
+      dev = dev->next;
     }
-#endif
 
   fp = sanei_config_open (GT68XX_CONFIG_FILE);
   if (!fp)
@@ -1321,9 +1312,46 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
     }
 
   fclose (fp);
+  return SANE_STATUS_GOOD;
+}
+
+/* -------------------------- SANE API functions ------------------------- */
+
+SANE_Status
+sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+{
+  SANE_Status status;
+
+  DBG_INIT ();
+#ifdef DBG_LEVEL
+  if (DBG_LEVEL > 0)
+    {
+      DBG (5, "sane_init: debug options are enabled, handle with care\n");
+      debug_options = SANE_TRUE;
+    }
+#endif
+  DBG (2, "SANE GT68xx backend version %d.%d build %d from %s\n", SANE_CURRENT_MAJOR,
+       V_MINOR, BUILD, PACKAGE_STRING);
+
+  if (version_code)
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
+
+  DBG (5, "sane_init: authorize %s null\n", authorize ? "!=" : "==");
+
+  sanei_usb_init ();
+
+  num_devices = 0;
+  first_dev = 0;
+  first_handle = 0;
+  devlist = 0;
+  new_dev = 0;
+  new_dev_len = 0;
+  new_dev_alloced = 0;
+
+  status = probe_gt68xx_devices ();
   DBG (5, "sane_init: exit\n");
 
-  return SANE_STATUS_GOOD;
+  return status;
 }
 
 void
@@ -1332,6 +1360,7 @@ sane_exit (void)
   GT68xx_Device *dev, *next;
 
   DBG (5, "sane_exit: start\n");
+  sanei_usb_exit();
   for (dev = first_dev; dev; dev = next)
     {
       next = dev->next;
@@ -1355,6 +1384,10 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
   DBG (5, "sane_get_devices: start: local_only = %s\n",
        local_only == SANE_TRUE ? "true" : "false");
 
+  /* hot-plug case : detection of newly connected scanners */
+  sanei_usb_scan_devices ();
+  probe_gt68xx_devices ();
+
   if (devlist)
     free (devlist);
 
@@ -1363,20 +1396,29 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
     return SANE_STATUS_NO_MEM;
 
   dev_num = 0;
-  for (dev = first_dev; dev_num < num_devices; dev = dev->next)
+  dev = first_dev;
+  while(dev!=NULL)
     {
       SANE_Device *sane_device;
 
-      sane_device = malloc (sizeof (*sane_device));
-      if (!sane_device)
-        return SANE_STATUS_NO_MEM;
-      sane_device->name = dev->file_name;
-      sane_device->vendor = dev->model->vendor;
-      sane_device->model = dev->model->model;
-      sane_device->type = strdup ("flatbed scanner");
-      devlist[dev_num++] = sane_device;
+      /* don't return devices that have been unplugged */
+      if(dev->missing==SANE_FALSE)
+        {
+          sane_device = malloc (sizeof (*sane_device));
+          if (!sane_device)
+            return SANE_STATUS_NO_MEM;
+          sane_device->name = dev->file_name;
+          sane_device->vendor = dev->model->vendor;
+          sane_device->model = dev->model->model;
+          sane_device->type = strdup ("flatbed scanner");
+          devlist[dev_num] = sane_device;
+          dev_num++;
+        }
+
+      /* next device */
+      dev = dev->next;
     }
-  devlist[dev_num++] = 0;
+  devlist[dev_num] = 0;
 
   *device_list = devlist;
 
@@ -1579,7 +1621,7 @@ sane_close (SANE_Handle handle)
   free (s->val[OPT_GRAY_MODE_COLOR].s);
   free (s->val[OPT_SOURCE].s);
   free (dev->file_name);
-  free (s->opt[OPT_RESOLUTION].constraint.word_list);
+  free ((void *)(size_t)s->opt[OPT_RESOLUTION].constraint.word_list);
 
   gt68xx_scanner_free (s);
   
@@ -2304,7 +2346,7 @@ sane_cancel (SANE_Handle handle)
   else
     {
       DBG (4, "sane_cancel: scan has not been initiated yet, "
-           "or it is allready aborted\n");
+           "or it is already aborted\n");
     }
 
   DBG (5, "sane_cancel: exit\n");
