@@ -45,7 +45,7 @@
 /* #define STUBS
 extern int sanei_debug_hp; */
 #define DEBUG_DECLARE_ONLY
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
@@ -59,7 +59,8 @@ extern int sanei_debug_hp; */
 
 #include "hp-handle.h"
 
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
+#include "../include/sane/sanei_thread.h"
 
 #include "hp-device.h"
 #include "hp-option.h"
@@ -73,11 +74,18 @@ struct hp_handle_s
     HpDevice		dev;
     SANE_Parameters	scan_params;
 
-    pid_t		reader_pid;
+    SANE_Pid		reader_pid;
+    int			child_forked; /* Flag if we used fork() or not */
     size_t		bytes_left;
-    int			pipefd;
+    int			pipe_read_fd;
+    sigset_t            sig_set;
 
     sig_atomic_t	cancelled;
+
+    /* These data are used by the child */
+    HpScsi      scsi;
+    HpProcessData procdata;
+    int			pipe_write_fd;
 };
 
 
@@ -87,52 +95,124 @@ hp_handle_isScanning (HpHandle this)
   return this->reader_pid != 0;
 }
 
-static SANE_Status
-hp_handle_startReader (HpHandle this, HpScsi scsi, HpProcessData *procdata)
+/*
+ * reader thread. Used when threads are used
+ */
+static int
+reader_thread (void *data)
 {
-  int	fds[2];
-  sigset_t 		sig_set, old_set;
+  struct hp_handle_s *this = (struct hp_handle_s *) data;
+  struct SIGACTION	act;
+  SANE_Status status;
+
+  DBG (1, "reader_thread: thread started\n"
+   "  parameters: scsi = 0x%08lx, pipe_write_fd = %d\n",
+          (long) this->scsi, this->pipe_write_fd);
+
+  memset(&act, 0, sizeof(act));
+  sigaction(SIGTERM, &act, 0);
+
+  DBG (1, "Starting sanei_hp_scsi_pipeout()\n");
+  status = sanei_hp_scsi_pipeout (this->scsi, this->pipe_write_fd,
+                                  &(this->procdata));
+  DBG (1, "sanei_hp_scsi_pipeout finished with %s\n", sane_strstatus (status));
+
+  close (this->pipe_write_fd);
+  this->pipe_write_fd = -1;
+  sanei_hp_scsi_destroy (this->scsi, 0);
+  return status;
+}
+
+/*
+ * reader process. Used when forking child.
+ */
+static int
+reader_process (void *data)
+{
+  struct hp_handle_s *this = (struct hp_handle_s *) data;
   struct SIGACTION	sa;
-  SANE_Status           status;
+  SANE_Status status;
 
-  assert(this->reader_pid == 0);
-  this->cancelled = 0;
-
-  if (pipe(fds))
-      return SANE_STATUS_IO_ERROR;
-
-  sigfillset(&sig_set);
-  sigprocmask(SIG_BLOCK, &sig_set, &old_set);
-
-  if ((this->reader_pid = fork()) != 0)
-    {
-      sigprocmask(SIG_SETMASK, &old_set, 0);
-      close(fds[1]);
-
-      if (this->reader_pid == -1)
-	{
-	  close(fds[0]);
-	  return SANE_STATUS_IO_ERROR;
-	}
-
-      this->pipefd = fds[0];
-      DBG(1, "start_reader: reader process %d started\n", this->reader_pid);
-      return SANE_STATUS_GOOD;
-    }
-
-  close(fds[0]);
+  /* Here we are in a forked child. The thread will not come up to here. */
+  /* Forked child must close read end of pipe */
+  close (this->pipe_read_fd);
+  this->pipe_read_fd = -1;
 
   memset(&sa, 0, sizeof(sa));
   sa.sa_handler = SIG_DFL;
   sigaction(SIGTERM, &sa, 0);
-  sigdelset(&sig_set, SIGTERM);
-  sigprocmask(SIG_SETMASK, &sig_set, 0);
+  sigdelset(&(this->sig_set), SIGTERM);
+  sigprocmask(SIG_SETMASK, &(this->sig_set), 0);
 
-  /* not closing fds[1] gives an infinite loop on Digital UNIX */
-  status = sanei_hp_scsi_pipeout(scsi,fds[1],procdata);
-  close (fds[1]);
-  DBG(3,"hp_handle_startReader: Exiting child (%s)\n",sane_strstatus(status));
-  _exit(status);
+  /* not closing writing end of pipe gives an infinite loop on Digital UNIX */
+  status = sanei_hp_scsi_pipeout (this->scsi, this->pipe_write_fd,
+                                  &(this->procdata));
+  close (this->pipe_write_fd);
+  this->pipe_write_fd = -1;
+  DBG(3,"reader_process: Exiting child (%s)\n",sane_strstatus(status));
+  return (status);
+}
+
+static SANE_Status
+hp_handle_startReader (HpHandle this, HpScsi scsi)
+{
+  int	fds[2];
+  sigset_t 		old_set;
+
+  assert(this->reader_pid == 0);
+  this->cancelled = 0;
+  this->pipe_write_fd = this->pipe_read_fd = -1;
+
+  if (pipe(fds))
+      return SANE_STATUS_IO_ERROR;
+
+  sigfillset(&(this->sig_set));
+  sigprocmask(SIG_BLOCK, &(this->sig_set), &old_set);
+
+  this->scsi = scsi;
+  this->pipe_write_fd = fds[1];
+  this->pipe_read_fd = fds[0];
+
+  /* Will childs be forked ? */
+  this->child_forked = sanei_thread_is_forked ();
+
+  /* Start a thread or fork a child. None of them will return here. */
+  /* Returning means to be in the parent or thread/fork failed */
+  this->reader_pid = sanei_thread_begin (this->child_forked ? reader_process :
+                                         reader_thread, (void *) this);
+  if (this->reader_pid != 0)
+    {
+      /* Here we are in the parent */
+      sigprocmask(SIG_SETMASK, &old_set, 0);
+
+      if ( this->child_forked )
+      { /* After fork(), parent must close writing end of pipe */
+        DBG(3, "hp_handle_startReader: parent closes write end of pipe\n");
+        close (this->pipe_write_fd);
+        this->pipe_write_fd = -1;
+      }
+
+      if (!sanei_thread_is_valid (this->reader_pid))
+	{
+          if ( !this->child_forked )
+          {
+            close (this->pipe_write_fd);
+            this->pipe_write_fd = -1;
+          }
+	  close (this->pipe_read_fd);
+          this->pipe_read_fd = -1;
+
+          DBG(1, "hp_handle_startReader: fork() failed\n");
+
+	  return SANE_STATUS_IO_ERROR;
+	}
+
+      DBG(1, "start_reader: reader process %ld started\n", (long) this->reader_pid);
+      return SANE_STATUS_GOOD;
+    }
+
+  DBG(3, "Unexpected return from sanei_thread_begin()\n");
+  return SANE_STATUS_INVAL;
 }
 
 static SANE_Status
@@ -146,13 +226,21 @@ hp_handle_stopScan (HpHandle this)
   if (this->reader_pid)
     {
       int info;
-      DBG(3, "hp_handle_stopScan: killing child (%d)\n", this->reader_pid);
-      kill(this->reader_pid, SIGTERM);
-      waitpid(this->reader_pid, &info, 0);
+      DBG(3, "hp_handle_stopScan: killing child (%ld)\n", (long) this->reader_pid);
+      if (this->child_forked)
+      {
+        kill(this->reader_pid, SIGTERM);
+        waitpid(this->reader_pid, &info, 0);
+      }
+      else
+      {
+        sanei_thread_kill (this->reader_pid);
+        sanei_thread_waitpid(this->reader_pid, &info);
+      }
       DBG(1, "hp_handle_stopScan: child %s = %d\n",
 	  WIFEXITED(info) ? "exited, status" : "signalled, signal",
 	  WIFEXITED(info) ? WEXITSTATUS(info) : WTERMSIG(info));
-      close(this->pipefd);
+      close(this->pipe_read_fd);
       this->reader_pid = 0;
 
       if ( !FAILED( sanei_hp_scsi_new(&scsi, this->dev->sanedev.name)) )
@@ -386,7 +474,7 @@ sanei_hp_handle_startScan (HpHandle this)
   SANE_Status	status;
   HpScsi	scsi;
   HpScl         scl;
-  HpProcessData procdata;
+  HpProcessData *procdata = &(this->procdata);
   int           adfscan;
 
   /* FIXME: setup preview mode stuff? */
@@ -403,9 +491,9 @@ sanei_hp_handle_startScan (HpHandle this)
 
   if (!FAILED(status))
      status = hp_handle_uploadParameters(this, scsi,
-                                         &(procdata.bits_per_channel),
-                                         &(procdata.invert),
-                                         &(procdata.out8));
+                                         &(procdata->bits_per_channel),
+                                         &(procdata->invert),
+                                         &(procdata->out8));
 
   if (FAILED(status))
     {
@@ -413,9 +501,9 @@ sanei_hp_handle_startScan (HpHandle this)
       return status;
     }
 
-  procdata.mirror_vertical =
+  procdata->mirror_vertical =
      sanei_hp_optset_mirror_vert (this->dev->options, this->data, scsi);
-  DBG(1, "start: %s to mirror image vertically\n", procdata.mirror_vertical ?
+  DBG(1, "start: %s to mirror image vertically\n", procdata->mirror_vertical ?
          "Request" : "No request" );
 
   scl = sanei_hp_optset_scan_type (this->dev->options, this->data);
@@ -527,7 +615,7 @@ sanei_hp_handle_startScan (HpHandle this)
     }
   }
 
-  DBG(1, "start: %s to mirror image vertically\n", procdata.mirror_vertical ?
+  DBG(1, "start: %s to mirror image vertically\n", procdata->mirror_vertical ?
          "Request" : "No request" );
 
   this->bytes_left = ( this->scan_params.bytes_per_line
@@ -536,35 +624,37 @@ sanei_hp_handle_startScan (HpHandle this)
   DBG(1, "start: %d pixels per line, %d bytes per line, %d lines high\n",
       this->scan_params.pixels_per_line, this->scan_params.bytes_per_line,
       this->scan_params.lines);
-  procdata.bytes_per_line = (int)this->scan_params.bytes_per_line;
-  if (procdata.out8)
+  procdata->bytes_per_line = (int)this->scan_params.bytes_per_line;
+  if (procdata->out8)
   {
-    procdata.bytes_per_line *= 2;
+    procdata->bytes_per_line *= 2;
     DBG(1,"(scanner will send %d bytes per line, 8 bit output forced)\n",
-        procdata.bytes_per_line);
+        procdata->bytes_per_line);
   }
-  procdata.lines = this->scan_params.lines;
+  procdata->lines = this->scan_params.lines;
 
   /* Wait for front-panel button push ? */
   status = sanei_hp_optset_start_wait(this->dev->options, this->data);
 
   if (status)   /* Wait for front button push ? Start scan in reader process */
   {
-    procdata.startscan = scl;
+    procdata->startscan = scl;
     status = SANE_STATUS_GOOD;
   }
   else
   {
-    procdata.startscan = 0;
+    procdata->startscan = 0;
     status = sanei_hp_scl_startScan(scsi, scl);
   }
 
   if (!FAILED( status ))
   {
-      status = hp_handle_startReader(this, scsi, &procdata);
+      status = hp_handle_startReader(this, scsi);
   }
 
-  sanei_hp_scsi_destroy(scsi,0);
+  /* Close SCSI-connection in forked environment */
+  if (this->child_forked)
+    sanei_hp_scsi_destroy(scsi,0);
 
   return status;
 }
@@ -598,7 +688,7 @@ sanei_hp_handle_read (HpHandle this, void * buf, size_t *lengthp)
   if (*lengthp > this->bytes_left)
       *lengthp = this->bytes_left;
 
-  if ((nread = read(this->pipefd, buf, *lengthp)) < 0)
+  if ((nread = read(this->pipe_read_fd, buf, *lengthp)) < 0)
     {
       *lengthp = 0;
       if (errno == EAGAIN)
@@ -654,9 +744,12 @@ sanei_hp_handle_cancel (HpHandle this)
   if (    (this->reader_pid)
        && (this->dev->compat & HP_COMPAT_OJ_1150C) )
   {
-     DBG(3,"sanei_hp_handle_cancel: send SIGTERM to child (%d)\n",
-         this->reader_pid);
-     kill(this->reader_pid, SIGTERM);
+     DBG(3,"sanei_hp_handle_cancel: send SIGTERM to child (%ld)\n",
+         (long) this->reader_pid);
+     if (this->child_forked)
+       kill(this->reader_pid, SIGTERM);
+     else
+       sanei_thread_kill(this->reader_pid);
   }
 }
 
@@ -673,7 +766,7 @@ sanei_hp_handle_setNonblocking (HpHandle this, hp_bool_t non_blocking)
       return SANE_STATUS_CANCELLED;
     }
 
-  if (fcntl(this->pipefd, F_SETFL, non_blocking ? O_NONBLOCK : 0) < 0)
+  if (fcntl(this->pipe_read_fd, F_SETFL, non_blocking ? O_NONBLOCK : 0) < 0)
       return SANE_STATUS_IO_ERROR;
 
   return SANE_STATUS_GOOD;
@@ -692,6 +785,6 @@ sanei_hp_handle_getPipefd (HpHandle this, SANE_Int *fd)
       return SANE_STATUS_CANCELLED;
     }
 
-  *fd = this->pipefd;
+  *fd = this->pipe_read_fd;
   return SANE_STATUS_GOOD;
 }
