@@ -43,8 +43,8 @@
 */
 
 /*
-   $Log: hp-scl.c,v $
-   Revision 1.15  2008-03-28 14:37:36  kitno-guest
+   $Log$
+   Revision 1.15  2008/03/28 14:37:36  kitno-guest
    add usleep to improve usb performance, from jim a t meyering d o t net
 
    Revision 1.14  2004-10-04 18:09:05  kig-guest
@@ -64,7 +64,7 @@
 #define STUBS
 extern int sanei_debug_hp;*/
 #define DEBUG_DECLARE_ONLY
-#include "sane/config.h"
+#include "../include/sane/config.h"
 #include "../include/lalloca.h"		/* Must be first */
 
 #ifdef HAVE_UNISTD_H
@@ -80,13 +80,13 @@ extern int sanei_debug_hp;*/
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include "sane/sanei_scsi.h"
-#include "sane/sanei_usb.h"
-#include "sane/sanei_pio.h"
+#include "../include/sane/sanei_scsi.h"
+#include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_pio.h"
 
 #include "hp.h"
 
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #include "hp-option.h"
 #include "hp-scsi.h"
@@ -339,7 +339,7 @@ hp_nonscsi_write (HpScsi this, hp_byte_t *data, size_t len, HpConnect connect)
 
 static SANE_Status
 hp_nonscsi_read (HpScsi this, hp_byte_t *data, size_t *len, HpConnect connect,
-  int UNUSEDARG isResponse)
+  int __sane_unused__ isResponse)
 
 {int n = -1;
  static int retries = -1;
@@ -905,7 +905,7 @@ hp_scsi_scl(HpScsi this, HpScl scl, int val)
 
   RETURN_IF_FAIL( hp_scsi_need(this, 10) );
 
-  /* Dont try to optimize SCL-commands like using <ESC>*a1b0c5T */
+  /* Don't try to optimize SCL-commands like using <ESC>*a1b0c5T */
   /* Some scanners have problems with it (e.g. HP Photosmart Photoscanner */
   /* with window position/extent, resolution) */
   count = sprintf((char *)this->bufp, "\033*%c%d%c", group, val, param);
@@ -1011,7 +1011,7 @@ hp_scsi_read (HpScsi this, void * dest, size_t *len, int isResponse)
 
 static int signal_caught = 0;
 
-static RETSIGTYPE
+static void
 signal_catcher (int sig)
 {
   DBG(1,"signal_catcher(sig=%d): old signal_caught=%d\n",sig,signal_caught);
@@ -1338,13 +1338,11 @@ static SANE_Status
 process_data (PROCDATA_HANDLE *ph, unsigned char *read_ptr, int nread)
 
 {int bytes_left;
- HpProcessData *procdata;
 
  if (nread <= 0) return SANE_STATUS_GOOD;
 
  if (ph == NULL) return SANE_STATUS_INVAL;
 
- procdata = &(ph->procdata);
  if ( ph->tmp_buf_len > 0 )  /* Something left ? */
  {
    bytes_left = ph->tmp_buf_size - ph->tmp_buf_len;
@@ -2083,7 +2081,7 @@ hp_scl_strerror (int errnum)
       case 1024: return "ADF Paper Jam";
       case 1025: return "Home Position Missing";
       case 1026: return "Paper Not Loaded";
-      default: return "??Unkown Error??";
+      default: return "??Unknown Error??";
       }
 }
 
