@@ -44,13 +44,13 @@
 */
 
 static char *hp_backend_version = "1.06";
-static char *hp_backend_revision = "$Revision: 1.22 $";
+static char *hp_backend_revision = "$Revision$";
 /* Changes:
 
    V 1.06:
-   $Log: hp.c,v $
-   Revision 1.22  2008-11-26 21:21:25  kitno-guest
-   * backend/*.[ch]: nearly every backend used V_MAJOR
+   $Log$
+   Revision 1.22  2008/11/26 21:21:25  kitno-guest
+   * backend/ *.[ch]: nearly every backend used V_MAJOR
    instead of SANE_CURRENT_MAJOR in sane_init()
    * backend/snapscan.c: remove EXPECTED_VERSION check
    since new SANE standard is forward compatible
@@ -232,17 +232,17 @@ static char *hp_backend_revision = "$Revision: 1.22 $";
 
 #define VERSIO                                8
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 #include "hp.h"
 
 #include <string.h>
 /* #include <sys/types.h> */
-/* #include <sane/sane.h> */
-#include "sane/sanei_config.h"
-#include "sane/sanei_backend.h"
-#include "sane/sanei_usb.h"
-#include "sane/sanei_thread.h"
-/* #include <sane/sanei_debug.h> */
+/* #include "../include/sane/sane.h" */
+#include "../include/sane/sanei_config.h"
+#include "../include/sane/sanei_backend.h"
+#include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_thread.h"
+/* #include "../include/sane/sanei_debug.h" */
 #include "hp-device.h"
 #include "hp-handle.h"
 
@@ -468,13 +468,11 @@ static void
 hp_device_info_remove (void)
 {
  HpDeviceInfoList  next, infolistelement = global.infolist;
- HpDeviceInfo *info;
 
  if (!global.is_up) return;
 
  while (infolistelement)
  {
-   info = &(infolistelement->info);
    next = infolistelement->next;
    sanei_hp_free (infolistelement);
    infolistelement = next;
@@ -811,7 +809,7 @@ hp_update_devlist (void)
  */
 
 SANE_Status
-sane_init (SANE_Int *version_code, SANE_Auth_Callback UNUSEDARG authorize)
+sane_init (SANE_Int *version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {SANE_Status status;
 
   DBG_INIT();
@@ -839,7 +837,7 @@ sane_exit (void)
 
 SANE_Status
 sane_get_devices (const SANE_Device ***device_list,
-                  SANE_Bool UNUSEDARG local_only)
+                  SANE_Bool __sane_unused__ local_only)
 {
   DBG(3, "sane_get_devices called\n");
 
