@@ -698,7 +698,6 @@ compute_first_gain_offset (int target, int max, int min, int *gain,
 static int
 write_gamma (HP4200_Scanner * s)
 {
-  SANE_Status status;
   int color;
   int i;
   unsigned char gamma[1024];
@@ -728,7 +727,7 @@ write_gamma (HP4200_Scanner * s)
       sanei_pv8630_write_byte (s->fd, PV8630_REPPADDRESS, 0x06);
       sanei_pv8630_prep_bulkread (s->fd, sizeof (read_gamma));
       to_read = sizeof (read_gamma);
-      status = sanei_usb_read_bulk (s->fd, read_gamma, &to_read);
+      sanei_usb_read_bulk (s->fd, read_gamma, &to_read);
       retval = memcmp (read_gamma, gamma, sizeof (read_gamma));
       if (retval != 0)
 	{
@@ -929,10 +928,13 @@ compute_dpd (HP4200_Scanner * s, int step_size, int line_end)
 		  s->hw_parms.pulse_duration + 1) +
 		 3 - s->hw_parms.num_tr_pulses));
 
+  if (tr == 0)
+    return 0;
+
   dpd = (((s->hw_parms.fsteps_25_speed * 4) +
 	  (s->hw_parms.fsteps_50_speed * 2) +
 	  s->hw_parms.steps_to_reverse) * 4 * step_size) % tr;
-  dpd = (tr == 0) ? 0 : tr - dpd;
+  dpd = tr - dpd;
 
   return dpd;
 }
@@ -1044,7 +1046,6 @@ do_coarse_calibration (HP4200_Scanner * s, struct coarse_t *coarse)
   int step_size;
   int ff_step_size;
   char steps_to_reverse;
-  char hdpi_div;
   char line_rate_color;
   int vdpi;			/* vertical dots per inch */
   int hdpi_code;
@@ -1082,7 +1083,6 @@ do_coarse_calibration (HP4200_Scanner * s, struct coarse_t *coarse)
 
   vdpi = 150;
   hdpi_code = 0;
-  hdpi_div = hdpi_mapping[hdpi_code];
   active_pixels_start = 0x40;
   line_end = 0x2ee0;
   s->mclk_div = 2;
@@ -1392,7 +1392,6 @@ do_fine_calibration (HP4200_Scanner * s, struct coarse_t *coarse)
   int vdpi;			/* vertical dots per inch */
   int hdpi_code;
   int calibrated;
-  int first_time;
   int lines_to_process;
 
   static char me[] = "do_fine_calibration";
@@ -1492,7 +1491,6 @@ do_fine_calibration (HP4200_Scanner * s, struct coarse_t *coarse)
   cache_write (s);
 
   calibrated = 0;
-  first_time = 1;
   cal_line = malloc (cal_line_size + 1024);
   average = malloc (sizeof (int) * line_length * 3);
   memset (average, 0, sizeof (int) * line_length * 3);
@@ -2434,7 +2432,7 @@ sane_exit (void)
 	}
       if (device->dev.name)
 	{
-	  free (device->dev.name);
+	  free ((void *) device->dev.name);
 	}
       free (device);
     }
