@@ -38,10 +38,11 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.
 
-   This file is part of a SANE backend for HP 4570/5550/5590/7650 Scanners
+   This file is part of a SANE backend for
+   HP ScanJet 4500C/4570C/5500C/5550C/5590/7650 Scanners
 */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -50,8 +51,11 @@
 # include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
-#include "sane/sanei_debug.h"
-#include "sane/sanei_usb.h"
+#include "byteorder.h"
+
+#include "../include/sane/sanei_debug.h"
+#include "../include/sane/sanei_usb.h"
+#include "../include/_stdint.h"
 #include "hp5590_low.h"
 
 /* Debug levels */
@@ -80,7 +84,7 @@ struct usb_in_usb_bulk_setup
   uint8_t	bRequest;
   uint8_t	bEndpoint;
   uint16_t	unknown;
-  uint16_t	wLength;
+  uint16_t	wLength;	/* MSB first */
   uint8_t 	pad;
 } __attribute__ ((packed));
 
@@ -88,9 +92,9 @@ struct usb_in_usb_bulk_setup
 struct usb_in_usb_ctrl_setup {
   uint8_t  bRequestType;
   uint8_t  bRequest;
-  uint16_t wValue;
-  uint16_t wIndex;
-  uint16_t wLength;
+  uint16_t wValue;		/* MSB first */
+  uint16_t wIndex;		/* MSB first */
+  uint16_t wLength;		/* LSB first */
 } __attribute__ ((packed));
 
 /* CORE status flag - ready or not */
@@ -129,12 +133,17 @@ struct bulk_read_state
  * SANE_STATUS_DEVICE_BUSY - otherwise
  */
 static SANE_Status
-hp5590_get_ack (SANE_Int dn)
+hp5590_get_ack (SANE_Int dn,
+		enum proto_flags proto_flags)
 {
   uint8_t 	status;
   SANE_Status	ret;
 
-  DBG (DBG_proc, "%s\n", __FUNCTION__);
+  /* Bypass reading acknowledge if the device doesn't need it */
+  if (proto_flags & PF_NO_USB_IN_USB_ACK)
+    return SANE_STATUS_GOOD;
+
+  DBG (DBG_proc, "%s\n", __func__);
 
   /* Check if USB-in-USB operation was accepted */
   ret = sanei_usb_control_msg (dn, USB_DIR_IN | USB_TYPE_VENDOR,
@@ -143,17 +152,17 @@ hp5590_get_ack (SANE_Int dn)
   if (ret != SANE_STATUS_GOOD)
     {
       DBG (DBG_err, "%s: USB-in-USB: error getting acknowledge\n",
-	   __FUNCTION__);
+	   __func__);
       return ret;
     }
 
-  DBG (DBG_usb, "%s: USB-in-USB: accepted\n", __FUNCTION__);
+  DBG (DBG_usb, "%s: USB-in-USB: accepted\n", __func__);
 
   /* Check if we received correct acknowledgement */
   if (status != 0x01)
     {
       DBG (DBG_err, "%s: USB-in-USB: not accepted (status %u)\n",
-	   __FUNCTION__, status);
+	   __func__, status);
       return SANE_STATUS_DEVICE_BUSY;
     }
 
@@ -171,20 +180,21 @@ hp5590_get_ack (SANE_Int dn)
  * SANE_STATUS_DEVICE_BUSY - otherwise
  */
 static SANE_Status
-hp5590_get_status (SANE_Int dn)
+hp5590_get_status (SANE_Int dn,
+		   __sane_unused__ enum proto_flags proto_flags)
 {
   uint8_t status;
   SANE_Status ret;
 
-  DBG (DBG_proc, "%s\n", __FUNCTION__);
+  DBG (DBG_proc, "%s\n", __func__);
 
   ret = sanei_usb_control_msg (dn, USB_DIR_IN | USB_TYPE_VENDOR,
-			       0x0c, 0x8e, 0x20,
+			       0x0c, 0x8e, 0x00,
 			       sizeof (status), &status);
   if (ret != SANE_STATUS_GOOD)
     {
       DBG (DBG_err, "%s: USB-in-USB: error getting device status\n",
-	   __FUNCTION__);
+	   __func__);
       return ret;
     }
 
@@ -192,7 +202,7 @@ hp5590_get_status (SANE_Int dn)
   if (status != 0x00)
     {
       DBG (DBG_err, "%s: USB-in-USB: got non-zero device status (status %u)\n",
-	   __FUNCTION__, status);
+	   __func__, status);
       return SANE_STATUS_DEVICE_BUSY;
     }
 
@@ -220,6 +230,7 @@ hp5590_get_status (SANE_Int dn)
  */
 static SANE_Status
 hp5590_control_msg (SANE_Int dn,
+		    enum proto_flags proto_flags,
 		    int requesttype, int request,
 		    int value, int index, unsigned char *bytes,
 		    int size, int core_flags)
@@ -233,7 +244,7 @@ hp5590_control_msg (SANE_Int dn,
   unsigned int 			needed_response;
 
   DBG (DBG_proc, "%s: USB-in-USB: core data: %s\n",
-       __FUNCTION__, core_flags & CORE_DATA ? "yes" : "no");
+       __func__, core_flags & CORE_DATA ? "yes" : "no");
 
   hp5590_low_assert (bytes != NULL);
 
@@ -246,9 +257,9 @@ hp5590_control_msg (SANE_Int dn,
       ctrl.bRequest = request;
       ctrl.wValue = htons (value);
       ctrl.wIndex = htons (index);
-      ctrl.wLength = size;
+      ctrl.wLength = htole16 (size);
 
-      DBG (DBG_usb, "%s: USB-in-USB: sending control msg\n", __FUNCTION__);
+      DBG (DBG_usb, "%s: USB-in-USB: sending control msg\n", __func__);
       /* Send USB-in-USB control message */
       ret = sanei_usb_control_msg (dn, USB_DIR_OUT | USB_TYPE_VENDOR,
 				   0x04, 0x8f, 0x00,
@@ -256,13 +267,12 @@ hp5590_control_msg (SANE_Int dn,
       if (ret != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_err, "%s: USB-in-USB: error sending control message\n",
-	       __FUNCTION__);
+	       __func__);
 	  return ret;
 	}
 
-      DBG (DBG_usb, "%s: USB-in-USB: checking acknowledge for control message\n",
-	   __FUNCTION__);
-      ret = hp5590_get_ack (dn);
+      /* USB-in-USB: checking acknowledge for control message */
+      ret = hp5590_get_ack (dn, proto_flags);
       if (ret != SANE_STATUS_GOOD)
 	return ret;
 
@@ -282,7 +292,7 @@ hp5590_control_msg (SANE_Int dn,
 				       0x90, 0x00, next_packet_size, ptr);
 	  if (ret != SANE_STATUS_GOOD)
 	    {
-	      DBG (DBG_err, "%s: USB-in-USB: error reading data\n", __FUNCTION__);
+	      DBG (DBG_err, "%s: USB-in-USB: error reading data\n", __func__);
 	      return ret;
 	    }
 
@@ -298,13 +308,12 @@ hp5590_control_msg (SANE_Int dn,
       if (ret != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_err, "%s: USB-in-USB: error confirming data reception\n",
-	       __FUNCTION__);
+	       __func__);
 	  return -1;
 	}
 
-      DBG (DBG_usb, "%s: USB-in-USB: checking if confirmation was acknowledged\n",
-	   __FUNCTION__);
-      ret = hp5590_get_ack (dn);
+      /* USB-in-USB: checking if confirmation was acknowledged */
+      ret = hp5590_get_ack (dn, proto_flags);
       if (ret != SANE_STATUS_GOOD)
 	return ret;
     }
@@ -318,9 +327,9 @@ hp5590_control_msg (SANE_Int dn,
       ctrl.bRequest = request;
       ctrl.wValue = htons (value);
       ctrl.wIndex = htons (index);
-      ctrl.wLength = size;
+      ctrl.wLength = htole16 (size);
 
-      DBG (DBG_usb, "%s: USB-in-USB: sending control msg\n", __FUNCTION__);
+      DBG (DBG_usb, "%s: USB-in-USB: sending control msg\n", __func__);
       /* Send USB-in-USB control message */
       ret = sanei_usb_control_msg (dn, USB_DIR_OUT | USB_TYPE_VENDOR,
 				   0x04, 0x8f, 0x00,
@@ -328,13 +337,12 @@ hp5590_control_msg (SANE_Int dn,
       if (ret != SANE_STATUS_GOOD)
 	{
 	  DBG (DBG_err, "%s: USB-in-USB: error sending control message\n",
-	       __FUNCTION__);
+	       __func__);
 	  return ret;
 	}
 
-      DBG (DBG_usb, "%s: USB-in-USB: checking acknowledge for control message\n",
-	   __FUNCTION__);
-      ret = hp5590_get_ack (dn);
+      /* USB-in-USB: checking acknowledge for control message */
+      ret = hp5590_get_ack (dn, proto_flags);
       if (ret != SANE_STATUS_GOOD)
 	return ret;
 
@@ -354,16 +362,15 @@ hp5590_control_msg (SANE_Int dn,
 				       0x8f, 0x00, next_packet_size, ptr);
 	  if (ret != SANE_STATUS_GOOD)
 	    {
-	      DBG (DBG_err, "%s: USB-in-USB: error sending data\n", __FUNCTION__);
+	      DBG (DBG_err, "%s: USB-in-USB: error sending data\n", __func__);
 	      return ret;
 	    }
 
 	  /* CORE data is acknowledged packet by packet */
 	  if (core_flags & CORE_DATA)
 	    {
-	      DBG (DBG_usb, "%s: USB-in-USB: checking if data was accepted\n",
-		   __FUNCTION__);
-	      ret = hp5590_get_ack (dn);
+	      /* USB-in-USB: checking if data was accepted */
+	      ret = hp5590_get_ack (dn, proto_flags);
 	      if (ret != SANE_STATUS_GOOD)
 		return ret;
 	    }
@@ -375,21 +382,20 @@ hp5590_control_msg (SANE_Int dn,
       /* Normal (non-CORE) data is acknowledged after its full transmission */
       if (!(core_flags & CORE_DATA))
 	{
-	  DBG (3, "%s: USB-in-USB: checking if data was accepted\n",
-	       __FUNCTION__);
-	  ret = hp5590_get_ack (dn);
+	  /* USB-in-USB: checking if data was accepted */
+	  ret = hp5590_get_ack (dn, proto_flags);
 	  if (ret != SANE_STATUS_GOOD)
 	    return ret;
 	}
 
       /* Getting  response after data transmission */
-      DBG (DBG_usb, "%s: USB-in-USB: getting response\n", __FUNCTION__);
+      DBG (DBG_usb, "%s: USB-in-USB: getting response\n", __func__);
       ret = sanei_usb_control_msg (dn, USB_DIR_IN | USB_TYPE_VENDOR,
 				   0x0c, 0x90, 0x00,
 				   sizeof (response), &response);
       if (ret != SANE_STATUS_GOOD)
 	{
-	  DBG (DBG_err, "%s: USB-in-USB: error getting response\n", __FUNCTION__);
+	  DBG (DBG_err, "%s: USB-in-USB: error getting response\n", __func__);
 	  return ret;
 	}
 
@@ -399,14 +405,14 @@ hp5590_control_msg (SANE_Int dn,
       needed_response = core_flags & CORE_BULK_OUT ? 0x24 : 0x00;
       if (response == needed_response)
 	DBG (DBG_usb, "%s: USB-in-USB: got correct response\n",
-	     __FUNCTION__);
+	     __func__);
 
       if (response != needed_response)
 	{
 	  DBG (DBG_err,
 	       "%s: USB-in-USB: invalid response received "
 	       "(needed %04x, got %04x)\n",
-	       __FUNCTION__, needed_response, response);
+	       __func__, needed_response, response);
 	  return SANE_STATUS_IO_ERROR;
 	}
 
@@ -415,7 +421,7 @@ hp5590_control_msg (SANE_Int dn,
 	{
 	  uint8_t bulk_flags = 0x24;
 	  DBG (DBG_usb, "%s: USB-in-USB: sending bulk flags\n",
-	       __FUNCTION__);
+	       __func__);
 
 	  ret = sanei_usb_control_msg (dn, USB_DIR_OUT | USB_TYPE_VENDOR,
 				       0x0c, 0x83, 0x00,
@@ -423,14 +429,12 @@ hp5590_control_msg (SANE_Int dn,
 	  if (ret != SANE_STATUS_GOOD)
 	    {
 	      DBG (DBG_err, "%s: USB-in-USB: error sending bulk flags\n",
-		   __FUNCTION__);
+		   __func__);
 	      return ret;
 	    }
 
-	  /* Check acknowledge for bulk flags transmission */
-	  DBG (DBG_usb, "%s: USB-in-USB: checking confirmation for bulk flags\n",
-	       __FUNCTION__);
-	  ret = hp5590_get_ack (dn);
+	  /* USB-in-USB: checking confirmation for bulk flags */
+	  ret = hp5590_get_ack (dn, proto_flags);
 	  if (ret != SANE_STATUS_GOOD)
 	    return ret;
 	}
@@ -453,7 +457,9 @@ hp5590_control_msg (SANE_Int dn,
  * all other SANE_Status values - otherwise
  */
 static SANE_Status
-hp5590_verify_last_cmd (SANE_Int dn, unsigned int cmd)
+hp5590_verify_last_cmd (SANE_Int dn,
+			enum proto_flags proto_flags,
+			unsigned int cmd)
 {
   uint16_t	verify_cmd;
   unsigned int	last_cmd;
@@ -461,16 +467,20 @@ hp5590_verify_last_cmd (SANE_Int dn, unsigned int cmd)
   SANE_Status 	ret;
 
   DBG (3, "%s: USB-in-USB: command verification requested\n",
-       __FUNCTION__);
+       __func__);
 
   /* Read last command along with CORE status */
-  ret = hp5590_control_msg (dn, USB_DIR_IN,
+  ret = hp5590_control_msg (dn,
+  			    proto_flags,
+  			    USB_DIR_IN,
 			    0x04, 0xc5, 0x00,
 			    (unsigned char *) &verify_cmd,
 			    sizeof (verify_cmd), CORE_NONE);
   if (ret != SANE_STATUS_GOOD)
     return ret;
 
+  verify_cmd = le16toh (verify_cmd); /* Response is LSB first */
+
   /* Last command - minor byte */
   last_cmd = verify_cmd & 0xff;
   /* CORE status - major byte */
@@ -479,17 +489,17 @@ hp5590_verify_last_cmd (SANE_Int dn, unsigned int cmd)
   /* Verify last command */
   DBG (DBG_usb, "%s: USB-in-USB: command verification %04x, "
        "last command: %04x, core status: %04x\n",
-       __FUNCTION__, verify_cmd, last_cmd, core_status);
+       __func__, verify_cmd, last_cmd, core_status);
   if ((cmd & 0x00ff) != last_cmd)
     {
       DBG (DBG_err, "%s: USB-in-USB: command verification failed: "
 	   "expected 0x%04x, got 0x%04x\n",
-	   __FUNCTION__, cmd, last_cmd);
+	   __func__, cmd, last_cmd);
       return SANE_STATUS_IO_ERROR;
     }
 
   DBG (DBG_usb, "%s: USB-in-USB: command verified successfully\n",
-       __FUNCTION__);
+       __func__);
 
   /* Return value depends on CORE status */
   return core_status & CORE_FLAG_NOT_READY ?
@@ -516,15 +526,18 @@ hp5590_verify_last_cmd (SANE_Int dn, unsigned int cmd)
  * all other SANE_Status values - otherwise
  */
 static SANE_Status
-hp5590_cmd (SANE_Int dn, unsigned int flags,
+hp5590_cmd (SANE_Int dn,
+	    enum proto_flags proto_flags,
+	    unsigned int flags,
 	    unsigned int cmd, unsigned char *data, unsigned int size,
 	    unsigned int core_flags)
 {
   SANE_Status ret;
 
-  DBG (3, "%s: USB-in-USB: command : %04x\n", __FUNCTION__, cmd);
+  DBG (3, "%s: USB-in-USB: command : %04x\n", __func__, cmd);
 
   ret = hp5590_control_msg (dn,
+  			    proto_flags,
 			    flags & CMD_IN ? USB_DIR_IN : USB_DIR_OUT,
 			    0x04, cmd, 0x00, data, size, core_flags);
   if (ret != SANE_STATUS_GOOD)
@@ -534,7 +547,7 @@ hp5590_cmd (SANE_Int dn, unsigned int flags,
   /* Verify last command if requested */
   if (flags & CMD_VERIFY)
     {
-      ret = hp5590_verify_last_cmd (dn, cmd);
+      ret = hp5590_verify_last_cmd (dn, proto_flags, cmd);
     }
 
   return ret;
@@ -555,7 +568,7 @@ hp5590_low_init_bulk_read_state (void **state)
 {
   struct bulk_read_state *bulk_read_state;
 
-  DBG (3, "%s: USB-in-USB: initializing bulk read state\n", __FUNCTION__);
+  DBG (3, "%s: USB-in-USB: initializing bulk read state\n", __func__);
 
   hp5590_low_assert (state != NULL);
 
@@ -569,7 +582,7 @@ hp5590_low_init_bulk_read_state (void **state)
   if (!bulk_read_state->buffer)
     {
       DBG (DBG_err, "%s: Memory allocation failed for %u bytes\n",
-	   __FUNCTION__, ALLOCATE_BULK_READ_PAGES * BULK_READ_PAGE_SIZE);
+	   __func__, ALLOCATE_BULK_READ_PAGES * BULK_READ_PAGE_SIZE);
       return SANE_STATUS_NO_MEM;
     }
   bulk_read_state->buffer_size = ALLOCATE_BULK_READ_PAGES
@@ -600,7 +613,7 @@ hp5590_low_free_bulk_read_state (void **state)
 {
   struct bulk_read_state *bulk_read_state;
 
-  DBG (3, "%s\n", __FUNCTION__);
+  DBG (3, "%s\n", __func__);
 
   hp5590_low_assert (state != NULL);
   /* Just return if NULL bulk read state was given */
@@ -609,7 +622,7 @@ hp5590_low_free_bulk_read_state (void **state)
 
   bulk_read_state = *state;
 
-  DBG (3, "%s: USB-in-USB: freeing bulk read state\n", __FUNCTION__);
+  DBG (3, "%s: USB-in-USB: freeing bulk read state\n", __func__);
 
   free (bulk_read_state->buffer);
   bulk_read_state->buffer = NULL;
@@ -630,7 +643,9 @@ hp5590_low_free_bulk_read_state (void **state)
  * state - pointer to initialized bulk read state structure
  */
 static SANE_Status
-hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
+hp5590_bulk_read (SANE_Int dn,
+		  enum proto_flags proto_flags,
+		  unsigned char *bytes, unsigned int size,
 		  void *state)
 {
   struct usb_in_usb_bulk_setup	ctrl;
@@ -641,7 +656,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
   struct bulk_read_state	*bulk_read_state;
   unsigned int 			bytes_until_buffer_end;
 
-  DBG (3, "%s\n", __FUNCTION__);
+  DBG (3, "%s\n", __func__);
 
   hp5590_low_assert (state != NULL);
   hp5590_low_assert (bytes != NULL);
@@ -650,7 +665,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
   if (bulk_read_state->initialized == 0)
     {
       DBG (DBG_err, "%s: USB-in-USB: bulk read state not initialized\n",
-	   __FUNCTION__);
+	   __func__);
       return SANE_STATUS_INVAL;
     }
 
@@ -670,7 +685,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
     {
       DBG (DBG_usb, "%s: USB-in-USB: not enough data in buffer available "
 	   "(available: %u, requested: %u)\n",
-	   __FUNCTION__, bulk_read_state->bytes_available, size);
+	   __func__, bulk_read_state->bytes_available, size);
 
       /* IMPORTANT! 'next_pages' means 'request and receive next_pages pages in
        * one bulk transfer request '. Windows driver uses 4 pages between each
@@ -688,7 +703,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
        */
       bulk_read_state->total_pages++;
       DBG (DBG_usb, "%s: USB-in-USB: total pages done: %u\n",
-	   __FUNCTION__, bulk_read_state->total_pages);
+	   __func__, bulk_read_state->total_pages);
 
       /* Send another bulk request for 'next_pages' before first
        * page or next necessary one
@@ -698,7 +713,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 	{
 	  /* Send bulk flags */
 	  DBG (DBG_usb, "%s: USB-in-USB: sending USB-in-USB bulk flags\n",
-	       __FUNCTION__);
+	       __func__);
 	  bulk_flags = 0x24;
 	  ret = sanei_usb_control_msg (dn, USB_DIR_OUT | USB_TYPE_VENDOR,
 				       0x0c, 0x83, 0x00,
@@ -706,14 +721,12 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 	  if (ret != SANE_STATUS_GOOD)
 	    {
 	      DBG (DBG_err, "%s: USB-in-USB: error sending bulk flags\n",
-		   __FUNCTION__);
+		   __func__);
 	      return ret;
 	    }
 
-	  /* Check acknowledge for bulk flags just sent */
-	  DBG (DBG_usb, "%s: USB-in-USB: checking confirmation for bulk flags\n",
-	       __FUNCTION__);
-	  ret = hp5590_get_ack (dn);
+	  /* USB-in-USB: checking confirmation for bulk flags\n" */
+	  ret = hp5590_get_ack (dn, proto_flags);
 	  if (ret != SANE_STATUS_GOOD)
 	    return ret;
 
@@ -725,7 +738,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 
 	  /* Send bulk read request */
 	  DBG (DBG_usb, "%s: USB-in-USB: sending control msg for bulk\n",
-	       __FUNCTION__);
+	       __func__);
 	  ret = sanei_usb_control_msg (dn, USB_DIR_OUT | USB_TYPE_VENDOR,
 				       0x04, 0x82, 0x00,
 				       sizeof (ctrl),
@@ -733,14 +746,12 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 	  if (ret != SANE_STATUS_GOOD)
 	    {
 	      DBG (DBG_err, "%s: USB-in-USB: error sending control msg\n",
-		   __FUNCTION__);
+		   __func__);
 	      return ret;
 	    }
 
-	  /* Check acknowledge for bulk read request */
-	  DBG (DBG_usb, "%s: USB-in-USB: checking if control msg was accepted\n",
-	       __FUNCTION__);
-	  ret = hp5590_get_ack (dn);
+	  /* USB-in-USB: checking if control msg was accepted */
+	  ret = hp5590_get_ack (dn, proto_flags); 
 	  if (ret != SANE_STATUS_GOOD)
 	    return ret;
 	}
@@ -750,13 +761,13 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
       if (bulk_read_state->buffer_size
 	  - bulk_read_state->bytes_available < next_portion)
 	{
-	  DBG (DBG_err, "%s: USB-in-USB: buffer too small\n", __FUNCTION__);
+	  DBG (DBG_err, "%s: USB-in-USB: buffer too small\n", __func__);
 	  return SANE_STATUS_NO_MEM;
 	}
 
       /* Bulk read next page */
       DBG (DBG_usb, "%s: USB-in-USB: bulk reading %lu bytes\n",
-	   __FUNCTION__, (u_long) next_portion);
+	   __func__, (u_long) next_portion);
       ret = sanei_usb_read_bulk (dn,
 				 bulk_read_state->buffer_in_ptr,
 				 &next_portion);
@@ -765,7 +776,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 	  if (ret == SANE_STATUS_EOF)
 	    return ret;
 	  DBG (DBG_err, "%s: USB-in-USB: error during bulk read: %s\n",
-	       __FUNCTION__, sane_strstatus (ret));
+	       __func__, sane_strstatus (ret));
 	  return ret;
 	}
 
@@ -774,7 +785,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 	{
 	  DBG (DBG_err, "%s: USB-in-USB: incomplete bulk read "
 	       "(requested %u bytes, got %lu bytes)\n",
-	       __FUNCTION__, BULK_READ_PAGE_SIZE, (u_long) next_portion);
+	       __func__, BULK_READ_PAGE_SIZE, (u_long) next_portion);
 	  return SANE_STATUS_IO_ERROR;
 	}
 
@@ -787,7 +798,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
 	  DBG (DBG_err,
 	       "%s: USB-in-USB: attempted to access over the end of buffer "
 	       "(in_ptr: %p, end_ptr: %p, ptr: %p, buffer size: %u\n",
-	       __FUNCTION__, bulk_read_state->buffer_in_ptr,
+	       __func__, bulk_read_state->buffer_in_ptr,
 	       bulk_read_state->buffer_end_ptr, bulk_read_state->buffer,
 	       bulk_read_state->buffer_size);
 	  return SANE_STATUS_NO_MEM;
@@ -797,7 +808,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
       if (bulk_read_state->buffer_in_ptr == bulk_read_state->buffer_end_ptr)
 	{
 	  DBG (DBG_usb, "%s: USB-in-USB: buffer wrapped while writing\n",
-	       __FUNCTION__);
+	       __func__);
 	  bulk_read_state->buffer_in_ptr = bulk_read_state->buffer;
 	}
 
@@ -808,7 +819,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
   /* Transfer requested amount of data to the caller */
   DBG (DBG_usb, "%s: USB-in-USB: data in bulk buffer is available "
        "(requested %u bytes, available %u bytes)\n",
-       __FUNCTION__, size, bulk_read_state->bytes_available);
+       __func__, size, bulk_read_state->bytes_available);
 
   /* Check for buffer pointer wrapping */
   bytes_until_buffer_end = bulk_read_state->buffer_end_ptr
@@ -816,13 +827,13 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
   if (bytes_until_buffer_end <= size)
     {
       /* First buffer part */
-      DBG (DBG_usb, "%s: USB-in-USB: reached bulk read buffer end\n", __FUNCTION__);
+      DBG (DBG_usb, "%s: USB-in-USB: reached bulk read buffer end\n", __func__);
       memcpy (bytes, bulk_read_state->buffer_out_ptr, bytes_until_buffer_end);
       bulk_read_state->buffer_out_ptr = bulk_read_state->buffer;
       /* And second part (if any) */
       if (bytes_until_buffer_end < size)
 	{
-	  DBG (DBG_usb, "%s: USB-in-USB: giving 2nd buffer part\n", __FUNCTION__);
+	  DBG (DBG_usb, "%s: USB-in-USB: giving 2nd buffer part\n", __func__);
 	  memcpy (bytes + bytes_until_buffer_end,
 		  bulk_read_state->buffer_out_ptr,
 		  size - bytes_until_buffer_end);
@@ -837,7 +848,7 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
       if (bulk_read_state->buffer_out_ptr == bulk_read_state->buffer_end_ptr)
 	{
 	  DBG (DBG_usb, "%s: USB-in-USB: buffer wrapped while reading\n",
-	       __FUNCTION__);
+	       __func__);
 	  bulk_read_state->buffer_out_ptr = bulk_read_state->buffer;
 	}
     }
@@ -862,7 +873,9 @@ hp5590_bulk_read (SANE_Int dn, unsigned char *bytes, unsigned int size,
  * all other SANE_Status value - otherwise 
  */
 static SANE_Status
-hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
+hp5590_bulk_write (SANE_Int dn,
+		   enum proto_flags proto_flags,
+		   int cmd, unsigned char *bytes,
 		   unsigned int size)
 {
   struct usb_in_usb_bulk_setup	ctrl;
@@ -873,7 +886,7 @@ hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
   unsigned char *ptr;
   size_t next_portion;
 
-  DBG (3, "%s: USB-in-USB: command: %04x, size %u\n", __FUNCTION__, cmd,
+  DBG (3, "%s: USB-in-USB: command: %04x, size %u\n", __func__, cmd,
        size);
 
   hp5590_low_assert (bytes != NULL);
@@ -885,8 +898,10 @@ hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
 
   /* Send bulk write request */
   DBG (3, "%s: USB-in-USB: total %u pages (each of %u bytes)\n",
-       __FUNCTION__, bulk_size.size, BULK_WRITE_PAGE_SIZE);
-  ret = hp5590_control_msg (dn, USB_DIR_OUT,
+       __func__, bulk_size.size, BULK_WRITE_PAGE_SIZE);
+  ret = hp5590_control_msg (dn,
+  			    proto_flags,
+  			    USB_DIR_OUT,
 			    0x04, cmd, 0,
 			    (unsigned char *) &bulk_size, sizeof (bulk_size),
 			    CORE_DATA | CORE_BULK_OUT);
@@ -904,7 +919,7 @@ hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
 	next_portion = len;
 
       DBG (3, "%s: USB-in-USB: next portion %lu bytes\n",
-	   __FUNCTION__, (u_long) next_portion);
+	   __func__, (u_long) next_portion);
 
       /* Prepare bulk write request */
       memset (&ctrl, 0, sizeof (ctrl));
@@ -919,16 +934,14 @@ hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
       if (ret != SANE_STATUS_GOOD)
 	return ret;
 
-      /* Check acknowledge for bulk write request */
-      DBG (DBG_usb, "%s: USB-in-USB: checking if command was accepted\n",
-	   __FUNCTION__);
-      ret = hp5590_get_ack (dn);
+      /* USB-in-USB: checking if command was accepted */
+      ret = hp5590_get_ack (dn, proto_flags);
       if (ret != SANE_STATUS_GOOD)
 	return ret;
 
       /* Write bulk data */
       DBG (3, "%s: USB-in-USB: bulk writing %lu bytes\n",
-	   __FUNCTION__, (u_long) next_portion);
+	   __func__, (u_long) next_portion);
       ret = sanei_usb_write_bulk (dn, ptr, &next_portion);
       if (ret != SANE_STATUS_GOOD)
 	{
@@ -936,7 +949,7 @@ hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
 	  if (ret == SANE_STATUS_EOF)
 	    break;
 	  DBG (DBG_err, "%s: USB-in-USB: error during bulk write: %s\n",
-	       __FUNCTION__, sane_strstatus (ret));
+	       __func__, sane_strstatus (ret));
 	  return ret;
 	}
 
@@ -946,7 +959,7 @@ hp5590_bulk_write (SANE_Int dn, int cmd, unsigned char *bytes,
     }
 
   /* Verify bulk command */
-  return hp5590_verify_last_cmd (dn, cmd);
+  return hp5590_verify_last_cmd (dn, proto_flags, cmd);
 }
 /* vim: sw=2 ts=8
  */
