@@ -64,7 +64,7 @@
 
 #ifdef HAVE_AUTHORIZATION
 #ifndef PATH_SEP
-#if defined(HAVE_OS2_H)
+#if defined(_WIN32) || defined(HAVE_OS2_H)
 # define PATH_SEP	"\\"
 #else
 # define PATH_SEP	"/"
@@ -1307,7 +1307,7 @@ set_option_dependencies(Microtek2_Scanner *,
 static SANE_Status
 shading_function(Microtek2_Scanner *, uint8_t *);
 
-static RETSIGTYPE
+static void
 signal_handler (int);
 
 static SANE_Status
