@@ -940,10 +940,10 @@ get_color_line_101x (Mustek_pp_Handle * dev, SANE_Byte * buf)
 	  wait_bank_change (dev, priv->bank_count, 1);
 	  reset_bank_count (dev);
 	  if (priv->ccd_line >= (priv->line_step >> SANE_FIXED_SCALE_SHIFT))
-	    priv->redline = ++priv->redline % priv->green_offs;
+	    priv->redline = (priv->redline + 1) % priv->green_offs;
 	  if (priv->ccd_line >=
 	      priv->blue_offs + (priv->line_step >> SANE_FIXED_SCALE_SHIFT))
-	    priv->blueline = ++priv->blueline % priv->blue_offs;
+	    priv->blueline = (priv->blueline + 1) % priv->blue_offs;
 	  continue;
 	}
 
@@ -979,7 +979,7 @@ get_color_line_101x (Mustek_pp_Handle * dev, SANE_Byte * buf)
 
 	}
 
-      priv->redline = ++priv->redline % priv->green_offs;
+      priv->redline = (priv->redline + 1) % priv->green_offs;
 
       if (priv->ccd_line >= priv->green_offs && gogreen)
 	{
@@ -1013,7 +1013,7 @@ get_color_line_101x (Mustek_pp_Handle * dev, SANE_Byte * buf)
 
       if (priv->ccd_line >=
 	  priv->blue_offs + (priv->line_step >> SANE_FIXED_SCALE_SHIFT))
-	priv->blueline = ++priv->blueline % priv->blue_offs;
+	priv->blueline = (priv->blueline + 1) % priv->blue_offs;
 
       if (gogreen)
 	{
@@ -1660,7 +1660,7 @@ ccd300_open (SANE_String port, SANE_Int caps, SANE_Int * fd)
 
   if (caps & ~(CAP_NOTHING | CAP_INVERT | CAP_LAMP_OFF))
     {
-      DBG (1, "ccd300_open: called with unknonw capabilities (%#02x)\n",
+      DBG (1, "ccd300_open: called with unknown capabilities (%#02x)\n",
 	   caps);
       return SANE_STATUS_INVAL;
     }
@@ -1816,7 +1816,7 @@ ccd300_config (SANE_Handle handle, SANE_String_Const optname,
     }
   else
     {
-      DBG (1, "ccd300_config: unkown option ``%s''", optname);
+      DBG (1, "ccd300_config: unknown option ``%s''", optname);
       return SANE_STATUS_INVAL;
     }
 
