@@ -1,8 +1,8 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2011-2016 Rolf Bensch <rolf at bensch hyphen online dot de>
    Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
-   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -57,6 +57,8 @@
 #include "pixma_common.h"
 #include "pixma_io.h"
 
+#include "../include/sane/sanei_usb.h"
+
 
 #ifdef __GNUC__
 # define UNUSED(v) (void) v
@@ -305,6 +307,32 @@ pixma_get_time (time_t * sec, uint32_t * usec)
     *usec = tv.tv_usec;
 }
 
+/* convert 24/48 bit RGB to 8/16 bit ir
+ *
+ * Formular: g = R
+ *           drop G + B
+ *
+ * sptr: source color scale buffer
+ * gptr: destination gray scale buffer
+ * c == 3: 24 bit RGB -> 8 bit ir
+ * c == 6: 48 bit RGB -> 16 bit ir
+ */
+uint8_t *
+pixma_r_to_ir (uint8_t * gptr, uint8_t * sptr, unsigned w, unsigned c)
+{
+  unsigned i;
+
+  /* PDBG (pixma_dbg (4, "*pixma_rgb_to_ir*****\n")); */
+
+  for (i = 0; i < w; i++)
+    {
+      *gptr++ = *sptr++;
+      if (c == 6) *gptr++ = *sptr++;            /* 48 bit RGB: high byte */
+      sptr += (c == 6) ? 4 : 2;                 /* drop G + B */
+    }
+  return gptr;
+}
+
 /* convert 24/48 bit RGB to 8/16 bit grayscale
  *
  * Formular: g = (R + G + B) / 3
@@ -357,7 +385,7 @@ pixma_binarize_line(pixma_scan_param_t * sp, uint8_t * dst, uint8_t * src, unsig
   unsigned char mask;
   uint8_t min, max;
 
-  /* PDBG (pixma_dbg (4, "*pixma_binarize_line***** src = %u, dst = %u, width = %u, c = %u, threshold = %u, thershold_curve = %u *****\n",
+  /* PDBG (pixma_dbg (4, "*pixma_binarize_line***** src = %u, dst = %u, width = %u, c = %u, threshold = %u, threshold_curve = %u *****\n",
                       src, dst, width, c, sp->threshold, sp->threshold_curve)); */
 
   /* 16 bit grayscale not supported */
@@ -623,13 +651,22 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
      immediatly answer with PIXMA_STATUS_BUSY.
 
      Is 8 seconds timeout enough? This affects ALL commands that use
-     pixma_cmd_transaction(). */
-  tmo = 8;
+     pixma_cmd_transaction(). Default value set in pixma_open(). */
+  tmo = s->rec_tmo;
   do
     {
       error = pixma_read (s->io, data, expected_len);
       if (error == PIXMA_ETIMEDOUT)
+      {
         PDBG (pixma_dbg (2, "No response yet. Timed out in %d sec.\n", tmo));
+
+#ifndef HAVE_SANEI_USB_SET_TIMEOUT
+        /* 1s timeout
+           Only needed, if sanei_usb_set_timeout() isn't available.
+           pixma_read() has an internal timeout of 1 sec. */
+        pixma_sleep (1000000);
+#endif
+      }
     }
   while (error == PIXMA_ETIMEDOUT && --tmo != 0);
   if (error < 0)
@@ -732,6 +769,7 @@ pixma_open (unsigned devnr, pixma_t ** handle)
   first_pixma = s;
 
   s->cfg = cfg;
+  s->rec_tmo = 8;               /* set receive timeout to 8 seconds */
   error = pixma_connect (devnr, &s->io);
   if (error < 0)
     {
@@ -810,6 +848,8 @@ pixma_scan (pixma_t * s, pixma_scan_param_t * sp)
 	     sp->xdpi, sp->ydpi, sp->x, sp->y, sp->w, sp->h);
   pixma_dbg (3, "  gamma_table=%p source=%d\n", sp->gamma_table, sp->source);
   pixma_dbg (3, "  threshold=%d threshold_curve=%d\n", sp->threshold, sp->threshold_curve);
+  pixma_dbg (3, "  adf-wait=%d\n", sp->adf_wait);
+  pixma_dbg (3, "  ADF page count: %d\n", sp->adf_pageid);
 #endif
 
   s->param = sp;
@@ -1003,10 +1043,10 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
     return PIXMA_EINVAL;
 
   /* flatbed: use s->cfg->xdpi
-   * TPU/ADF: use s->cfg->ext_max_dpi, if configured with dpi value */
+   * TPU/ADF: use s->cfg->adftpu_max_dpi, if configured with dpi value */
   cfg_xdpi = ((sp->source == PIXMA_SOURCE_FLATBED
-               || s->cfg->ext_max_dpi == 0) ? s->cfg->xdpi
-                                            : s->cfg->ext_max_dpi);
+               || s->cfg->adftpu_max_dpi == 0) ? s->cfg->xdpi
+                                               : s->cfg->adftpu_max_dpi);
 
   if (pixma_check_dpi (sp->xdpi, cfg_xdpi) < 0 ||
       pixma_check_dpi (sp->ydpi, s->cfg->ydpi) < 0)
@@ -1021,9 +1061,11 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
   if (s->ops->check_param (s, sp) < 0)
     return PIXMA_EINVAL;
 
-  /* FIXME: I assume the same minimum width and height for every model. */
-  CLAMP2 (sp->x, sp->w, 13, s->cfg->width, sp->xdpi);
-  CLAMP2 (sp->y, sp->h, 8, s->cfg->height, sp->ydpi);
+  /* FIXME: I assume the same minimum width and height for every model.
+   * new scanners need minimum 16 px height
+   * minimum image size: 16 px x 16 px */
+  CLAMP2 (sp->x, sp->w, 16, s->cfg->width, sp->xdpi);
+  CLAMP2 (sp->y, sp->h, 16, s->cfg->height, sp->ydpi);
 
   switch (sp->source)
     {
