@@ -1,6 +1,7 @@
 /* SANE - Scanner Access Now Easy.
  * For limitations, see function sanei_usb_get_vendor_product().
 
+   Copyright (C) 2011-2016 Rolf Bensch <rolf at bensch hyphen online dot de>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
    This file is part of the SANE package.
@@ -128,11 +129,11 @@ static const struct pixma_config_t *lookup_scanner(const char *makemodel,
                   (match[strlen(cfg->model)] == '\0') ||
                   (match[strlen(cfg->model)] == '-'))
                 {
-                  pixma_dbg (13, "Scanner model found: Name %s(%s) matches %s\n", cfg->model, cfg->name, makemodel);
+                  pixma_dbg (3, "Scanner model found: Name %s(%s) matches %s\n", cfg->model, cfg->name, makemodel);
                   return cfg;
                 }
             }
-          pixma_dbg (13, "Name %s(%s) does not match %s\n", cfg->model, cfg->name, makemodel);
+          pixma_dbg (20, "Scanner model %s(%s) not found, giving up! %s\n", cfg->model, cfg->name, makemodel);
        }
     }
   return NULL;
@@ -164,6 +165,7 @@ attach_bjnp (SANE_String_Const devname,  SANE_String_Const makemodel,
 {
   scanner_info_t *si;
   const pixma_config_t *cfg;
+  SANE_Status error;
 
   si = (scanner_info_t *) calloc (1, sizeof (*si));
   if (!si)
@@ -172,14 +174,18 @@ attach_bjnp (SANE_String_Const devname,  SANE_String_Const makemodel,
   if (!si->devname)
     return SANE_STATUS_NO_MEM;
   if ((cfg = lookup_scanner(makemodel, pixma_devices)) == (struct pixma_config_t *)NULL)
-    return SANE_STATUS_INVAL;
-  si->cfg = cfg;
-  sprintf(si->serial, "%s_%s", cfg->model, serial);
-  si -> interface = INT_BJNP;
-  si->next = first_scanner;
-  first_scanner = si;
-  nscanners++;
-  return SANE_STATUS_GOOD;
+    error = SANE_STATUS_INVAL;
+  else
+    {
+      si->cfg = cfg;
+      sprintf(si->serial, "%s_%s", cfg->model, serial);
+      si -> interface = INT_BJNP;
+      si->next = first_scanner;
+      first_scanner = si;
+      nscanners++;
+      error = SANE_STATUS_GOOD;
+    }
+  return error;
 }
 
 static void
@@ -232,9 +238,8 @@ read_serial_number (scanner_info_t * si)
 {
   uint8_t unicode[2 * (PIXMA_MAX_ID_LEN - 9) + 2];
   uint8_t ddesc[18];
-  int i, len, iSerialNumber;
+  int iSerialNumber;
   SANE_Int usb;
-  SANE_Status status;
   char *serial = si->serial;
 
   u16tohex (si->cfg->vid, serial);
@@ -248,6 +253,9 @@ read_serial_number (scanner_info_t * si)
   iSerialNumber = ddesc[16];
   if (iSerialNumber != 0)
     {
+      int i, len;
+      SANE_Status status;
+
       /*int iSerialNumber = ddesc[16];*/
       /* Read the first language code. Assumed that there is at least one. */
       if (get_string_descriptor (usb, 0, 0, 4, unicode) != SANE_STATUS_GOOD)
@@ -567,8 +575,8 @@ pixma_wait_interrupt (pixma_io_t * io, void *buf, unsigned size, int timeout)
 #endif
       error = map_error (sanei_usb_read_int (io->dev, buf, &count));
     }
-  if (error == PIXMA_EIO
-      || (io->interface == INT_USB && error == PIXMA_EOF))
+  if (error == PIXMA_EIO ||
+      (io->interface == INT_BJNP && error == PIXMA_EOF))     /* EOF is a bjnp timeout error! */
     error = PIXMA_ETIMEDOUT;	/* FIXME: SANE doesn't have ETIMEDOUT!! */
   if (error == 0)
     error = count;
