@@ -1,7 +1,7 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2011-2016 Rolf Bensch <rolf at bensch hyphen online dot de>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
-   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -324,8 +324,8 @@ read_image_block (pixma_t * s, uint8_t * data)
     return count;
   if (count == IMAGE_BLOCK_SIZE)
     {
-      int error;
-      if ((error = pixma_read (s->io, &temp, 0)) < 0)
+      int error = pixma_read (s->io, &temp, 0);
+      if (error < 0)
         {
           PDBG (pixma_dbg
           (1, "WARNING: reading zero-length packet failed %d\n", error));
@@ -488,6 +488,7 @@ calc_component_shifting (pixma_t * s)
 	  return s->param->ydpi / 75;
 	}
       /* never reached */
+      break;
 
     case MP750_PID:
     case MP780_PID:
@@ -881,7 +882,7 @@ mp750_finish_scan (pixma_t * s)
     case state_scanning:
     case state_warmup:
       error = abort_session (s);
-      if (abort_session (s) == PIXMA_ECANCELED)
+      if (error == PIXMA_ECANCELED)
 	read_error_info (s, NULL, 0);
       /* fall through */
     case state_finished:
@@ -955,7 +956,8 @@ static const pixma_scan_ops_t pixma_mp750_ops = {
 	0,                     /* iface */		\
 	&pixma_mp750_ops,      /* ops */		\
 	dpi, 2*(dpi),          /* xdpi, ydpi */		\
-        0, 0,                  /* ext_min_dpi & ext_max_dpi not used in this subdriver */ \
+        0, 0,                  /* adftpu_min_dpi & adftpu_max_dpi not used in this subdriver */ \
+        0, 0,                  /* tpuir_min_dpi & tpuir_max_dpi not used in this subdriver */   \
 	637, 877,              /* width, height */	\
         PIXMA_CAP_GRAY|PIXMA_CAP_EVENTS|cap             \
 }
