@@ -1,6 +1,6 @@
 /* SANE - Scanner Access Now Easy.
 
- Copyright (C) 2011-2015 Rolf Bensch <rolf at bensch hyphen online dot de>
+ Copyright (C) 2011-2016 Rolf Bensch <rolf at bensch hyphen online dot de>
  Copyright (C) 2007-2009 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
  Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
@@ -1009,6 +1009,7 @@ static int query_status (pixma_t * s)
   return error;
 }
 
+#if 0
 static int send_time (pixma_t * s)
 {
   /* Why does a scanner need a time? */
@@ -1025,6 +1026,7 @@ static int send_time (pixma_t * s)
   PDBG(pixma_dbg (3, "Sending time: '%s'\n", (char *) data));
   return pixma_exec (s, &mp->cb);
 }
+#endif
 
 /* TODO: Simplify this function. Read the whole data packet in one shot. */
 static int read_image_block (pixma_t * s, uint8_t * header, uint8_t * data)
@@ -1157,7 +1159,10 @@ static int handle_interrupt (pixma_t * s, int timeout)
   {
     /* More than one event can be reported at the same time. */
     if (buf[3] & 1)
-      send_time (s);    /* FIXME: some scanners hang here */
+      /* FIXME: This function makes trouble with a lot of scanners
+      send_time (s);
+       */
+      PDBG (pixma_dbg (1, "WARNING:send_time() disabled!\n"));
     if (buf[9] & 2)
       query_status (s);
 
