@@ -7,7 +7,7 @@
  *  @brief Main defines for the USB devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2013 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -40,6 +40,19 @@
  * - 0.50 - cleanup
  *        - removed obsolete _WAF_BLACKFINE
  *        - added MODEL_CANON_LIDE25
+ * - 0.51 - added _WAF_MISC_IO_BUTTONS plus _BUTTON stuff
+ *        - added _WAF_USE_ALT_DESC
+ *        - added DEVCAPSFLAG_SheetFed
+ *        - added dpi_thresh and lineend to motor structure
+ * - 0.52 - added MODEL_QSCAN
+ *        - added MODEL_QSCAN_A6 (thanks to Hiroshi Miura)
+ *        - changed DCapsDef, lamp -> misc_io
+ *        - bPCB is now ushort to be "missused" by non Plustek
+ *          devices (as threshhold for resetting sensor order)
+ *        - added _WAF_LOFF_ON_START and _WAF_ONLY_8BIT
+ *        - added MODEL_TSCAN_A4
+ *        - added attribute packed for data access structs
+ *        - added _WAF_INC_DARKTGT
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -113,20 +126,26 @@
 #define _HIBYTE(x)  ((u_char)((x) >> 8))
 
 #define _HILO2WORD(x)   ((u_short)x.bHi * 256U + x.bLo)
+#define _LOHI2WORD(x)   ((u_short)x.bLo * 256U + x.bHi)
+
 #define _PHILO2WORD(x)  ((u_short)x->bHi * 256U + x->bLo)
+#define _PLOHI2WORD(x)  ((u_short)x->bLo * 256U + x->bHi)
+
+#define PACKED8  __attribute__ ((packed,aligned(1)))
+#define PACKED16 __attribute__ ((packed,aligned(2)))
 
 /* useful for RGB-values */
 typedef struct {
 	u_char Red;
 	u_char Green;
 	u_char Blue;
-} RGBByteDef;
+} PACKED8 RGBByteDef;
 
 typedef struct {
 	u_short Red;
 	u_short Green;
 	u_short Blue;
-} RGBUShortDef;
+} PACKED16 RGBUShortDef;
 
 typedef struct {
 	u_long Red;
@@ -136,22 +155,22 @@ typedef struct {
 
 typedef struct {
 	u_char a_bColor[3];
-} ColorByteDef;
+} PACKED8 ColorByteDef;
 
 typedef struct {
 	u_char bHi;
 	u_char bLo;
-} HiLoDef;
+} PACKED8 HiLoDef;
 
 typedef union {
 	HiLoDef HiLo[3];
 	u_short Colors[3];
-} ColorWordDef;
+} PACKED16 ColorWordDef;
 
 typedef union {
 	HiLoDef HiLo;
 	u_short Mono;
-} MonoWordDef;
+} PACKED16 MonoWordDef;
 
 typedef union {
 
@@ -164,7 +183,7 @@ typedef union {
 	RGBUShortDef *pw_rgb;
 	HiLoDef      *philo;
 
-} AnyPtr;
+} __attribute__ ((aligned(4)))  AnyPtr;
 
 typedef struct {
 	unsigned short x;
@@ -181,9 +200,6 @@ typedef struct {
 
 #define GAIN_Target                     65535UL
 
-#define DRAM_UsedByAsic8BitMode         216             /* in KB */
-#define DRAM_UsedByAsic16BitMode        196 /*192*/     /* in KB */
-
 /** Chip-types */
 typedef enum _CHIPSET
 {
@@ -229,7 +245,8 @@ enum _DEVCAPSFLAG
 	DEVCAPSFLAG_Negative = 0x0004,
 	DEVCAPSFLAG_TPA      = 0x0006,
 	DEVCAPSFLAG_Adf      = 0x0008,
-	DEVCAPSFLAG_LargeTPA = 0x0010
+	DEVCAPSFLAG_LargeTPA = 0x0010,
+	DEVCAPSFLAG_SheetFed = 0x0020
 };
 
 /** to allow some workarounds */
@@ -244,7 +261,13 @@ enum _WORKAROUNDS
 	_WAF_SKIP_FINE          = 0x00000020, /* skip the fine calbration        */
 	_WAF_SKIP_WHITEFINE     = 0x00000040, /* skip the fine white calbration  */
 	_WAF_BIN_FROM_COLOR     = 0x00000080, /* generate binary & gray images   */
-	_WAF_GRAY_FROM_COLOR    = 0x00000100  /* from color scans                */
+	_WAF_GRAY_FROM_COLOR    = 0x00000100, /* from color scans                */
+	_WAF_MISC_IO_BUTTONS    = 0x00000200, /* special handling for buttons    */
+	_WAF_USE_ALT_DESC       = 0x00000400, /* use alternate manufacturer      */
+	_WAF_RESET_SO_TO_RGB    = 0x00000800, /* set sensororder to RGB(CIS only)*/
+	_WAF_LOFF_ON_START      = 0x00001000, /* switch lamp off before scanning */
+	_WAF_ONLY_8BIT          = 0x00002000, /* scanner allows only 8 bit modes */
+	_WAF_INC_DARKTGT        = 0x00004000  /* allow tgt darklevel adjustment  */
 };
 
 /** for lamps connected to the misc I/O pins*/
@@ -259,14 +282,44 @@ enum _LAMPS
 	_MIO6   = 0x0020
 };
 
+#define _PSENSE_SHIFT 24
+#define _PSENSE_MASK  0xFF000000
+#define _BUTTON_SHIFT 16
+#define _BUTTON_MASK  0xFF0000
+#define _TPA_SHIFT    8
+#define _TPA_MASK     0xFF00
+
+enum _BUTTONS
+{
+	_NO_BUTTON = 0,
+	_PORT0     = ((_MIO1 | _MIO2) << _BUTTON_SHIFT),
+	_PORT1     = ((_MIO3 | _MIO4) << _BUTTON_SHIFT),
+	_PORT2     = ((_MIO5 | _MIO6) << _BUTTON_SHIFT)
+};
+
+enum _PAPER_SENSE
+{
+	_PS_INP1     = (0x01  << _PSENSE_SHIFT),
+	_PS_INP2     = (0x02  << _PSENSE_SHIFT),
+	_PS_INP_MIO1 = (_MIO1 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO2 = (_MIO2 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO3 = (_MIO3 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO4 = (_MIO4 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO5 = (_MIO5 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO6 = (_MIO6 << (_PSENSE_SHIFT+2))
+};
+
 /** for encoding a misc I/O register as TPA */
-#define _TPA(register) ((u_long)(register << 16))
+#define _TPA(register) ((u_long)(register << _TPA_SHIFT))
 
 /** Mask to check for available TPA */
-#define _HAS_TPA(flag) (flag & 0xFFFF0000)
+#define _HAS_TPA(flag) (flag & _TPA_MASK)
 
 /** Get the TPA misc I/O register */
-#define _GET_TPALAMP(flag) (flag >> 16)
+#define _GET_TPALAMP(flag) ((flag >> _TPA_SHIFT) & 0xFF)
+
+/** Get the Papersense port*/
+#define _GET_PAPERSENSE_PORT(flag) ((flag >> _PSENSE_SHIFT) & 0xFF)
 
 /** motor types */
 typedef enum
@@ -284,6 +337,10 @@ typedef enum
 	MODEL_CANON_LIDE25,   /**< for CanoScan LiDE25          */
 	MODEL_UMAX,           /**< for UMAX 3400/3450           */
 	MODEL_UMAX1200,       /**< for UMAX 5400                */
+	MODEL_TSCAN,          /**< for Syscan TravelScan A6     */
+	MODEL_TSCAN_A4,       /**< for Syscan TravelScan A4     */
+	MODEL_QSCAN,          /**< for PandP Q-Scan A4          */
+	MODEL_QSCAN_A6,       /**< for PandP Q-Scan A6          */
 	MODEL_LAST
 } eModelDef;
 
@@ -325,10 +382,11 @@ enum SCANFLAG
 	SCANFLAG_DWORDBoundary  = 0x00020000,
 	SCANFLAG_RightAlign     = 0x00040000,
 	SCANFLAG_StillModule    = 0x00080000, 
-	SCANFLAG_StartScan      = 0x40000000,
-	SCANFLAG_Scanning       = 0x20020000,
 	SCANFLAG_Pseudo48       = 0x08000000,
-	SCANFLAG_SampleY        = 0x04000000
+	SCANFLAG_SampleY        = 0x04000000,
+	SCANFLAG_Calibration    = 0x10000000,
+	SCANFLAG_Scanning       = 0x20020000,
+	SCANFLAG_StartScan      = 0x40000000
 };
 
 typedef	struct Origins
@@ -347,7 +405,6 @@ typedef struct SrcAttr
 	short  DarkShadOrgY;    /**< if the device has a dark calibration strip  */
 	XY     Size;            /**< Scanning width/height, in 300 DPI base.     */
 	XY     MinDpi;          /**< Minimum dpi supported for scanning          */
-	u_char bMinDataType;    /**< Minimum data type supports                  */
 
 } SrcAttrDef;
 
@@ -364,10 +421,10 @@ typedef struct DevCaps
 	u_char     bSensorDistance; /**< CCD Color distance                      */
 	u_char     bButtons;        /**< Number of buttons                       */
 	u_char     bCCD;            /**< CCD ID                                  */
-	u_char     bPCB;            /**< PCB ID                                  */
+	u_short    bPCB;            /**< PCB ID/or threshold (only CIS)          */
 	u_long     workaroundFlag;  /**< Flag to allow special work arounds, see */
 	                            /*   _WORKAROUNDS                            */
-	u_long     lamp;            /**< for lamp: loword: normal, hiword: tpa   */
+	u_long     misc_io;         /**< for lamp, papersense and buttons        */
 
 } DCapsDef;
 
@@ -466,7 +523,7 @@ typedef struct HWDefault
 
 	eChipDef            chip;           /* chiptype               */
 	eModelDef           motorModel;     /* to identify used motor */
-	double              gamma;          /* gamma setting          */
+	double              gamma;          /* default gamma setting  */
 } HWDef;
 
 /** device description during runtime
@@ -580,7 +637,7 @@ typedef struct ScanDef
 	/** Image processing routine according to the scan mode  */
 	void (*pfnProcess)(struct Plustek_Device*);
 
-	u_char* pScanBuffer;      /**< our scan buffer */
+	u_long* pScanBuffer;      /**< our scan buffer */
 
 	u_long  dwLinesPerScanBufs;
 	u_long  dwNumberOfScanBufs;
@@ -635,14 +692,17 @@ typedef struct {
 	u_char  pwm_duty_fast;  /**< PWM duty during fast movement */
 	u_char  mclk_fast;      /**< MCLK during fast movement     */
 
-    /**
-     * here we define some ranges for better supporting
-     * non-Plustek devices with it's different hardware
-     * we can set the MCLK and the motor PWM stuff for color
-     * and gray modes (8bit and 14/16bit modes)
-     *    0    1     2     3     4     5     6      7     8      9
-     * <= 75 <=100 <=150 <=200 <=300 <=400 <=600 <= 800 <=1200 <=2400DPI
-     */
+	u_short dpi_thresh;
+	u_short lineend;
+
+	/**
+	 * here we define some ranges for better supporting
+	 * non-Plustek devices with it's different hardware
+	 * we can set the MCLK and the motor PWM stuff for color
+	 * and gray modes (8bit and 14/16bit modes)
+	 *    0    1     2     3     4     5     6      7     8      9
+	 * <= 75 <=100 <=150 <=200 <=300 <=400 <=600 <= 800 <=1200 <=2400DPI
+	 */
 	MDef   motor_sets[_MAX_CLK];	/**< motor PWM settings during scan      */
 	double color_mclk_8[_MAX_CLK];  /**< MCLK settings for color scan        */
 	double color_mclk_16[_MAX_CLK]; /**< MCLK settings for color (16bit) scan*/
