@@ -40,11 +40,11 @@
 */
 
 /*
-	$Id: ricoh.c,v 1.5 2003/09/12 16:10:33 hmg-guest Exp $
+	$Id$
         This file implements a SANE backend for Ricoh flatbed scanners.  
 */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <limits.h>
 #include <stdlib.h>
@@ -55,25 +55,25 @@
 #include <unistd.h>
 #include <ctype.h>
 
-#include "sane/sane.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_scsi.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
 
 #define BACKEND_NAME ricoh
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sanei_config.h"
+#include "../include/sane/sanei_config.h"
 #define RICOH_CONFIG_FILE "ricoh.conf"
 
 #include "ricoh.h"
 
-#define MM_PER_INCH	25.4
 #define MAX(a,b)	((a) > (b) ? (a) : (b))
 
+static const SANE_Device **devlist = NULL;
 static int num_devices = 0;
 static Ricoh_Device *first_dev = NULL;
 static Ricoh_Scanner *first_handle = NULL;
@@ -143,15 +143,15 @@ attach (const char *devnam, Ricoh_Device ** devp)
     }
 
   if (ibuf.devtype != 6
-      || strncmp (ibuf.vendor, "RICOH", 5) != 0
-      || (strncmp (ibuf.product, "IS50", 4) != 0
-	  && strncmp (ibuf.product, "IS60", 4) != 0))
+      || strncmp ((char *)ibuf.vendor, "RICOH", 5) != 0
+      || (strncmp ((char *)ibuf.product, "IS50", 4) != 0
+	  && strncmp ((char *)ibuf.product, "IS60", 4) != 0))
     {
       DBG (1, "attach: device doesn't look like the Ricoh scanner I know\n");
       sanei_scsi_close (fd);
       return (SANE_STATUS_INVAL);
     }
-  is50 = (strncmp (ibuf.product, "IS50", 4) == 0);
+  is50 = (strncmp ((char *)ibuf.product, "IS50", 4) == 0);
 
   DBG (3, "attach: sending TEST_UNIT_READY\n");
   status = test_unit_ready (fd);
@@ -222,11 +222,13 @@ attach (const char *devnam, Ricoh_Device ** devp)
 
   dev->sane.name = strdup (devnam);
   dev->sane.vendor = "RICOH";
-  str = malloc (16 + 1);
-  memset (str, 0, sizeof (str));
-  strncpy (str, ibuf.product, sizeof(ibuf.product));
-  strncpy (str + sizeof(ibuf.revision), ibuf.revision, sizeof(ibuf.revision));
-  str[sizeof(ibuf.product) + sizeof(ibuf.revision)] = '\0';
+  str = malloc (sizeof(ibuf.product) + sizeof(ibuf.revision) + 1);
+  if (str)
+    {
+      str[0] = '\0';
+      strncat (str, (char *)ibuf.product, sizeof(ibuf.product));
+      strncat (str, (char *)ibuf.revision, sizeof(ibuf.revision));
+    }
   dev->sane.model = str;
   dev->sane.type = "flatbed scanner";
 
@@ -327,6 +329,7 @@ init_options (Ricoh_Scanner * s)
 
   s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  s->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
   s->val[OPT_NUM_OPTS].w = NUM_OPTIONS;
 
@@ -493,7 +496,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 #endif
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open(RICOH_CONFIG_FILE);
   if (fp)
@@ -535,6 +538,9 @@ sane_exit (void)
       free ((void *) dev->sane.model);
       free (dev);
     }
+  
+  if (devlist)
+    free (devlist);
 
   DBG (11, "<< sane_exit\n");
 }
@@ -542,7 +548,6 @@ sane_exit (void)
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
-  static const SANE_Device **devlist = 0;
   Ricoh_Device *dev;
   int i;
 
@@ -755,7 +760,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
         }
 
       mode = s->val[OPT_MODE].s;
-      if (strcmp (mode, "Lineart") == 0 || strcmp (mode, "Halftone") == 0)
+      if (strcmp (mode, SANE_VALUE_SCAN_MODE_LINEART) == 0 || strcmp (mode, SANE_VALUE_SCAN_MODE_HALFTONE) == 0)
         {
           s->params.format = SANE_FRAME_GRAY;
           s->params.bytes_per_line = s->params.pixels_per_line / 8;
@@ -763,7 +768,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
           s->params.pixels_per_line = s->params.bytes_per_line * 8;
           s->params.depth = 1;
         }
-      else /* if (strcmp (mode, "Gray") == 0) */
+      else /* if (strcmp (mode, SANE_VALUE_SCAN_MODE_GRAY) == 0) */
         {
           s->params.format = SANE_FRAME_GRAY;
           s->params.bytes_per_line = s->params.pixels_per_line;
@@ -819,15 +824,15 @@ sane_start (SANE_Handle handle)
   s->brightness = s->val[OPT_BRIGHTNESS].w;
   s->contrast = s->val[OPT_CONTRAST].w;
   s->bpp = s->params.depth;
-  if (strcmp (mode_str, "Lineart") == 0) 
+  if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_LINEART) == 0) 
     {
       s->image_composition = RICOH_BINARY_MONOCHROME;
     }
-  else if (strcmp (mode_str, "Halftone") == 0)
+  else if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_HALFTONE) == 0)
     {
       s->image_composition = RICOH_DITHERED_MONOCHROME;
     }
-  else if (strcmp (mode_str, "Gray") == 0)
+  else if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_GRAY) == 0)
     {
       s->image_composition = RICOH_GRAYSCALE;
     }
