@@ -73,7 +73,7 @@ Start: 2.4.2001
 #include "../include/sane/saneopts.h"
 #include "../include/sane/sanei_usb.h"
 
-#undef HAVE_LIBUSB
+#undef HAVE_LIBUSB_LEGACY
 
 /* prevent inclusion of scantool.h */
 #define SCANTOOL_H
@@ -186,6 +186,7 @@ InitOptions(TInstance *this)
 	case optCount:
 	  pdesc->title  =SANE_TITLE_NUM_OPTIONS;
 	  pdesc->desc   =SANE_DESC_NUM_OPTIONS;
+	  pdesc->type   =SANE_TYPE_INT;
 	  pdesc->cap    =SANE_CAP_SOFT_DETECT;
 	  pval->w       =(SANE_Word)optLast;
 	  break;
@@ -391,9 +392,9 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authCB)
   DBG(DEBUG_VERBOSE,"SM3600 init\n");
   if (version_code)
    {
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
     DBG(DEBUG_VERBOSE,"SM3600 version: %x\n",
-    	SANE_VERSION_CODE(V_MAJOR, V_MINOR, BUILD));
+    	SANE_VERSION_CODE(SANE_CURRENT_MAJOR, V_MINOR, BUILD));
    }
 
   pdevFirst=NULL;
@@ -430,13 +431,11 @@ sane_exit (void)
 
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list,
-		  SANE_Bool local_only)
+		  SANE_Bool __sane_unused__ local_only)
 {
   TDevice *dev;
   int i;
 
-  local_only = TRUE; /* Avoid compile warning */
-
   if (devlist) free (devlist);
 
   devlist = malloc ((num_devices + 1) * sizeof (devlist[0]));
@@ -457,7 +456,6 @@ sane_open (SANE_String_Const devicename, SANE_Handle *handle)
 {
   TDevice    *pdev;
   TInstance  *this;
-  SANE_Status rc;
   DBG(DEBUG_VERBOSE,"opening %s\n",devicename);
   if (devicename[0]) /* selected */
     {
@@ -487,8 +485,6 @@ DBG(DEBUG_VERBOSE,"%s<>%s\n",devicename, pdev->sane.name);
   if (sanei_usb_open (devicename, &this->hScanner) != SANE_STATUS_GOOD)
     return SetError (this, SANE_STATUS_IO_ERROR, "cannot open scanner device");
 
-  rc = SANE_STATUS_GOOD;
-
   this->quality=fast;
   return InitOptions(this);
 }
