@@ -44,10 +44,10 @@
    If you do not wish that, delete this exception notice.
  
    This file is a component of the implementation of a backend for many
-   of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
+   of the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-sources.h,v 1.5 2001/12/17 22:51:50 oliverschwartz Exp $
+/* $Id$
    SnapScan backend scan data sources */
 
 #ifndef SNAPSCAN_SOURCES_H
@@ -87,7 +87,7 @@ static SANE_Status Source_init (Source *pself,
 #endif
 
 /*
- * $Log: snapscan-sources.h,v $
+ * $Log$
  * Revision 1.5  2001/12/17 22:51:50  oliverschwartz
  * Update to snapscan-20011212 (snapscan 1.4.3)
  *
