@@ -37,7 +37,7 @@
  */
 
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -50,14 +50,14 @@
 #include <unistd.h>
 #include <sys/time.h>
 
-#include "_stdint.h"
+#include "../include/_stdint.h"
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_scsi.h"
-#include "sane/sanei_thread.h"
-#include "sane/sanei_config.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
+#include "../include/sane/sanei_thread.h"
+#include "../include/sane/sanei_config.h"
 
 /* For timeval... */
 #ifdef DEBUG
@@ -66,7 +66,7 @@
 
 
 #define BACKEND_NAME	tamarack
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #include "tamarack.h"
 
@@ -76,15 +76,18 @@
 
 #define TAMARACK_CONFIG_FILE "tamarack.conf"
 
-#define MM_PER_INCH	25.4
 
+static const SANE_Device **devlist = NULL;
 static int num_devices;
 static Tamarack_Device *first_dev;
 static Tamarack_Scanner *first_handle;
 
 static const SANE_String_Const mode_list[] =
   {
-    "Thresholded", "Dithered", "Gray", "Color",
+    SANE_VALUE_SCAN_MODE_LINEART,
+    SANE_VALUE_SCAN_MODE_HALFTONE,
+    SANE_VALUE_SCAN_MODE_GRAY,
+    SANE_VALUE_SCAN_MODE_COLOR,
     0
   };
 
@@ -467,7 +470,7 @@ do_cancel (Tamarack_Scanner *s)
 
   do_eof (s);
 
-  if (s->reader_pid != -1)
+  if (sanei_thread_is_valid (s->reader_pid))
     {
       int exit_status;
 
@@ -948,12 +951,14 @@ sane_exit (void)
     free ((void *) dev->sane.model);
     free (dev);
   }
+  
+  if (devlist)
+    free (devlist);
 }
 
 SANE_Status
 sane_get_devices (const SANE_Device ***device_list, SANE_Bool local_only)
 {
-  static const SANE_Device **devlist = 0;
   Tamarack_Device *dev;
   int i;
 
@@ -1068,13 +1073,13 @@ sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
 
 static int make_mode (char *mode)
 {
-    if (strcmp (mode, "Thresholded") == 0)
+    if (strcmp (mode, SANE_VALUE_SCAN_MODE_LINEART) == 0)
       return THRESHOLDED;
-    if (strcmp (mode, "Dithered") == 0)
+    if (strcmp (mode, SANE_VALUE_SCAN_MODE_HALFTONE) == 0)
       return DITHERED;
-    else if (strcmp (mode, "Gray") == 0)
+    else if (strcmp (mode, SANE_VALUE_SCAN_MODE_GRAY) == 0)
       return GREYSCALE;
-    else if (strcmp (mode, "Color") == 0)
+    else if (strcmp (mode, SANE_VALUE_SCAN_MODE_COLOR) == 0)
       return TRUECOLOR;
 
     return -1;
@@ -1230,7 +1235,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 #endif
 
 
-	  if (strcmp (val, "Thresholded") == 0) 
+	  if (strcmp (val, SANE_VALUE_SCAN_MODE_LINEART) == 0) 
 	    s->opt[OPT_THRESHOLD].cap  &= ~SANE_CAP_INACTIVE;
 	  else {
 	    s->opt[OPT_BRIGHTNESS].cap &= ~SANE_CAP_INACTIVE;
@@ -1241,9 +1246,9 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    s->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
 
 	  if (s->val[OPT_CUSTOM_GAMMA].w) {
-	    if (strcmp (val, "Gray") == 0)
+	    if (strcmp (val, SANE_VALUE_SCAN_MODE_GRAY) == 0)
 	      s->opt[OPT_GAMMA_VECTOR].cap &= ~SANE_CAP_INACTIVE;
-	    else if (strcmp (val, "Color") == 0) {
+	    else if (strcmp (val, SANE_VALUE_SCAN_MODE_COLOR) == 0) {
 	      s->opt[OPT_GAMMA_VECTOR].cap   &= ~SANE_CAP_INACTIVE;
 	      s->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
 	      s->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
@@ -1437,7 +1442,7 @@ sane_cancel (SANE_Handle handle)
 {
   Tamarack_Scanner *s = handle;
 
-  if (s->reader_pid != -1)
+  if (sanei_thread_is_valid (s->reader_pid))
     sanei_thread_kill (s->reader_pid);
   s->scanning = SANE_FALSE;
 }
