@@ -42,7 +42,7 @@
 */
 
 /* 
-	$Id: teco3.h,v 1.5 2005/07/07 11:55:42 fzago-guest Exp $
+	$Id$
 */
 
 /* Commands supported by the scanner. */
@@ -228,7 +228,6 @@ getbitfield (unsigned char *pageaddr, int mask, int shift)
 
 /*--------------------------------------------------------------------------*/
 
-#define MM_PER_INCH     25.4
 #define mmToIlu(mm) (((mm) * 300) / MM_PER_INCH)
 #define iluToMm(ilu) (((ilu) * MM_PER_INCH) / 300)
 
@@ -291,9 +290,9 @@ struct scanners_supported
 
 /*--------------------------------------------------------------------------*/
 
-#define BLACK_WHITE_STR		SANE_I18N("Black & White")
-#define GRAY_STR			SANE_I18N("Grayscale")
-#define COLOR_STR			SANE_I18N("Color")
+#define BLACK_WHITE_STR		SANE_VALUE_SCAN_MODE_LINEART
+#define GRAY_STR		SANE_VALUE_SCAN_MODE_GRAY
+#define COLOR_STR		SANE_VALUE_SCAN_MODE_COLOR
 
 /*--------------------------------------------------------------------------*/
 
