@@ -7,6 +7,8 @@
  * - 0.01 - initial version
  * - 0.02 - cleanup
  *        - added lampTimer stuff
+ *        - fixed issue for Genius device - reported by
+ *          Jose Alberto Reguero <jareguero@telefonica.net>
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -336,7 +338,6 @@ static void u12hw_SetGeneralRegister( U12_Device *dev )
 static void u12hw_SetupPreviewCondition( U12_Device *dev )
 {
 	int       i, c;
-	u_long    channel;
 	SANE_Byte rb[100];
 
 	DBG( _DBG_INFO, "u12_SetupPreviewCondition()\n" );
@@ -387,7 +388,6 @@ static void u12hw_SetupPreviewCondition( U12_Device *dev )
 	else
 		dev->scan.bFifoSelect = REG_GFIFOOFFSET;
 
-	channel = _BLUE_FULLSIZE << 16;
 	dev->regs.RD_BufFullSize = _SIZE_BLUEFIFO;
 
 	dev->regs.RD_LineControl    = _LOBYTE(dev->shade.wExposure);
@@ -804,7 +804,9 @@ static void u12hw_CancelSequence( U12_Device *dev )
 	u12io_DataToRegister( dev, REG_MODELCONTROL, 0x1a );
 
 	u12hw_PutToIdleMode( dev );
-	u12io_SoftwareReset( dev );
+
+	if( strcmp( dev->usbId, "0x0458-0x2004" ))
+		u12io_SoftwareReset( dev );
 
 	u12motor_PositionModuleToHome( dev );
 
