@@ -278,19 +278,19 @@ static void sig_chldhandler( int signo )
 
 /** signal handler to kill the child process
  */
-static RETSIGTYPE reader_process_sigterm_handler( int signo )
+static void reader_process_sigterm_handler( int signo )
 {
 	DBG( _DBG_PROC, "(SIG) reader_process: terminated by signal %d\n", signo );
 	_exit( SANE_STATUS_GOOD );
 }
 
-static RETSIGTYPE usb_reader_process_sigterm_handler( int signo )
+static void usb_reader_process_sigterm_handler( int signo )
 {
 	DBG( _DBG_PROC, "(SIG) reader_process: terminated by signal %d\n", signo );
 	cancelRead = SANE_TRUE;
 }
 
-static RETSIGTYPE sigalarm_handler( int signo )
+static void sigalarm_handler( int signo )
 {
 	_VAR_NOT_USED( signo );
 	DBG( _DBG_PROC, "ALARM!!!\n" );
@@ -391,7 +391,7 @@ static SANE_Status do_cancel( U12_Scanner *scanner, SANE_Bool closepipe )
 
 	scanner->scanning = SANE_FALSE;
 
-	if( scanner->reader_pid != -1 ) {
+	if( sanei_thread_is_valid (scanner->reader_pid) ) {
 
 		DBG( _DBG_PROC, ">>>>>>>> killing reader_process <<<<<<<<\n" );
 
@@ -1731,7 +1731,7 @@ SANE_Status sane_start( SANE_Handle handle )
 	
 	cancelRead = SANE_FALSE;
 	
-	if( s->reader_pid == -1 ) {
+	if( !sanei_thread_is_valid (s->reader_pid) ) {
 		DBG( _DBG_ERROR, "ERROR: could not start reader task\n" );
 		s->scanning = SANE_FALSE;
 		u12if_close( dev );
