@@ -193,15 +193,8 @@ enum Umax_PP_Configure_Option
   NUM_CFG_OPTIONS
 };
 
-#if (!defined __GNUC__ || __GNUC__ < 2 || \
-     __GNUC_MINOR__ < (defined __cplusplus ? 6 : 4))
-
-#define __PRETTY_FUNCTION__	"umax_pp"
-
-#endif
-
 #define DEBUG()		DBG(4, "%s(v%d.%d.%d-%s): line %d: debug exception\n", \
-			  __PRETTY_FUNCTION__, SANE_CURRENT_MAJOR, V_MINOR,	\
+			  __func__, SANE_CURRENT_MAJOR, V_MINOR,	\
 			  UMAX_PP_BUILD, UMAX_PP_STATE, __LINE__)
 
 #endif /* umax_pp_h */
