@@ -290,6 +290,13 @@ init_options (V4L_Scanner * s)
   s->val[OPT_MODE].s = strdup (mode_list[0]);
   if (!s->val[OPT_MODE].s)
     return SANE_STATUS_NO_MEM;
+  s->opt[OPT_MODE].size = 1; /* '\0' */
+  for (i = 0; mode_list[i] != 0; ++i)
+    {
+      int len = strlen(mode_list[i]) + 1;
+      if (s->opt[OPT_MODE].size < len)
+        s->opt[OPT_MODE].size = len;
+    }
 
   /* channel */
   s->opt[OPT_CHANNEL].name = "channel";
@@ -305,6 +312,13 @@ init_options (V4L_Scanner * s)
     return SANE_STATUS_NO_MEM;
   if (s->channel[0] == 0 || s->channel[1] == 0)
     s->opt[OPT_CHANNEL].cap |= SANE_CAP_INACTIVE;
+  s->opt[OPT_CHANNEL].size = 1; /* '\0' */
+  for (i = 0; s->channel[i] != 0; ++i)
+    {
+      int len = strlen(s->channel[i]) + 1;
+      if (s->opt[OPT_CHANNEL].size < len)
+        s->opt[OPT_CHANNEL].size = len;
+    }
 
   /* "Geometry" group: */
   s->opt[OPT_GEOMETRY_GROUP].title = "Geometry";
@@ -481,13 +495,12 @@ sane_exit (void)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   V4L_Device *dev;
   int i;
 
   DBG (5, "sane_get_devices\n");
-  local_only = SANE_TRUE;	/* Avoid compile warning */
 
   if (devlist)
     free (devlist);
@@ -948,7 +961,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 SANE_Status
 sane_start (SANE_Handle handle)
 {
-  int len, loop;
+  int len;
   V4L_Scanner *s;
   char data;
 
@@ -984,6 +997,7 @@ sane_start (SANE_Handle handle)
     }
   else
     {
+      int loop;
       s->is_mmap = SANE_TRUE;
       DBG (3,
 	   "sane_start: mmap frame, buffersize: %d bytes, buffers: %d, offset 0 %d\n",
@@ -1032,6 +1046,7 @@ sane_start (SANE_Handle handle)
   /* v4l1 actually returns BGR when we ask for RGB, so convert it */
   if (s->pict.palette == VIDEO_PALETTE_RGB24)
     {
+      __u32 loop;
       DBG (3, "sane_start: converting from BGR to RGB\n");
       for (loop = 0; loop < (s->window.width * s->window.height * 3); loop += 3)
         {
@@ -1116,22 +1131,15 @@ sane_cancel (SANE_Handle handle)
 
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool non_blocking)
 {
-  /* Avoid compile warning */
-  handle = 0;
-
   if (non_blocking == SANE_FALSE)
     return SANE_STATUS_GOOD;
   return SANE_STATUS_UNSUPPORTED;
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
-  /* Avoid compile warning */
-  handle = 0;
-  fd = 0;
-
   return SANE_STATUS_UNSUPPORTED;
 }
