@@ -82,6 +82,7 @@
 #include <pwd.h>
 #include <grp.h>
 
+#include "lgetopt.h"
 
 #if defined(HAVE_SYS_POLL_H) && defined(HAVE_POLL)
 # include <sys/poll.h>
@@ -175,6 +176,10 @@ static AvahiSimplePoll *avahi_poll = NULL;
 static AvahiEntryGroup *avahi_group = NULL;
 #endif /* WITH_AVAHI */
 
+#ifdef HAVE_SYSTEMD
+#include <systemd/sd-daemon.h>
+#endif
+
 
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
@@ -193,16 +198,19 @@ static AvahiEntryGroup *avahi_group = NULL;
 #endif
 
 #ifdef ENABLE_IPV6
-# define SANE_IN6_IS_ADDR_LOOPBACK(a) \
+# ifndef IN6_IS_ADDR_LOOPBACK
+# define IN6_IS_ADDR_LOOPBACK(a) \
         (((const uint32_t *) (a))[0] == 0                                   \
          && ((const uint32_t *) (a))[1] == 0                                \
          && ((const uint32_t *) (a))[2] == 0                                \
          && ((const uint32_t *) (a))[3] == htonl (1)) 
-
-#define SANE_IN6_IS_ADDR_V4MAPPED(a) \
+# endif
+# ifndef IN6_IS_ADDR_V4MAPPED
+# define IN6_IS_ADDR_V4MAPPED(a) \
 ((((const uint32_t *) (a))[0] == 0)                                 \
  && (((const uint32_t *) (a))[1] == 0)                              \
  && (((const uint32_t *) (a))[2] == htonl (0xffff))) 
+# endif
 #endif /* ENABLE_IPV6 */
 
 #ifndef MAXHOSTNAMELEN
@@ -244,6 +252,7 @@ static int num_handles;
 static int debug;
 static int run_mode;
 static Handle *handle;
+static char *bind_addr;
 static union
 {
   int w;
@@ -783,7 +792,7 @@ check_host (int fd)
 #ifdef ENABLE_IPV6
   sin6 = &remote_address.sin6;
 
-  if (SANE_IN6_IS_ADDR_V4MAPPED (sin6->sin6_addr.s6_addr))
+  if (IN6_IS_ADDR_V4MAPPED ((struct in6_addr *)sin6->sin6_addr.s6_addr))
     {
       DBG (DBG_DBG, "check_host: detected an IPv4-mapped address\n");
       remote_ipv4 = remote_ip + 7;
@@ -840,7 +849,7 @@ check_host (int fd)
 	break;
 #ifdef ENABLE_IPV6
       case AF_INET6:
-	if (SANE_IN6_IS_ADDR_LOOPBACK (sin6->sin6_addr.s6_addr))
+	if (IN6_IS_ADDR_LOOPBACK ((struct in6_addr *)sin6->sin6_addr.s6_addr))
 	  {
 	    DBG (DBG_MSG,
 		 "check_host: remote host is IN6_LOOPBACK: access granted\n");
@@ -1425,7 +1434,7 @@ start_scan (Wire * w, int h, SANE_Start_Reply * reply)
   SANE_Handle be_handle;
   int fd, len;
   in_port_t data_port;
-  int ret;
+  int ret = -1;
 
   be_handle = handle[h].handle;
 
@@ -1983,6 +1992,38 @@ process_request (Wire * w)
 	    return 1;
 	  }
 
+        /* Addresses CVE-2017-6318 (#315576, Debian BTS #853804) */
+        /* This is done here (rather than in sanei/sanei_wire.c where
+         * it should be done) to minimize scope of impact and amount
+         * of code change.
+         */
+        if (w->direction == WIRE_DECODE
+            && req.value_type == SANE_TYPE_STRING
+            && req.action     == SANE_ACTION_GET_VALUE)
+          {
+            if (req.value)
+              {
+                /* FIXME: If req.value contains embedded NUL
+                 *        characters, this is wrong but we do not have
+                 *        access to the amount of memory allocated in
+                 *        sanei/sanei_wire.c at this point.
+                 */
+                w->allocated_memory -= (1 + strlen (req.value));
+                free (req.value);
+              }
+            req.value = malloc (req.value_size);
+            if (!req.value)
+              {
+                w->status = ENOMEM;
+                DBG (DBG_ERR,
+                     "process_request: (control_option) "
+                     "h=%d (%s)\n", req.handle, strerror (w->status));
+                return 1;
+              }
+            memset (req.value, 0, req.value_size);
+            w->allocated_memory += req.value_size;
+          }
+
 	can_authorize = 1;
 
 	memset (&reply, 0, sizeof (reply));	/* avoid leaking bits */
@@ -2804,13 +2845,13 @@ do_bindings (int *nfds, struct pollfd **fds)
   hints.ai_flags = AI_PASSIVE;
   hints.ai_socktype = SOCK_STREAM;
 
-  err = getaddrinfo (NULL, SANED_SERVICE_NAME, &hints, &res);
+  err = getaddrinfo (bind_addr, SANED_SERVICE_NAME, &hints, &res);
   if (err)
     {
       DBG (DBG_WARN, "do_bindings: \" %s \" service unknown on your host; you should add\n", SANED_SERVICE_NAME);
       DBG (DBG_WARN, "do_bindings:      %s %d/tcp saned # SANE network scanner daemon\n", SANED_SERVICE_NAME, SANED_SERVICE_PORT);
       DBG (DBG_WARN, "do_bindings: to your /etc/services file (or equivalent). Proceeding anyway.\n");
-      err = getaddrinfo (NULL, SANED_SERVICE_PORT_S, &hints, &res);
+      err = getaddrinfo (bind_addr, SANED_SERVICE_PORT_S, &hints, &res);
       if (err)
 	{
 	  DBG (DBG_ERR, "do_bindings: getaddrinfo() failed even with numeric port: %s\n", gai_strerror (err));
@@ -2888,7 +2929,10 @@ do_bindings (int *nfds, struct pollfd **fds)
   memset (&sin, 0, sizeof (sin));
 
   sin.sin_family = AF_INET;
-  sin.sin_addr.s_addr = INADDR_ANY;
+  if(bind_addr)
+    sin.sin_addr.s_addr = inet_addr(bind_addr);
+  else
+    sin.sin_addr.s_addr = INADDR_ANY;
   sin.sin_port = port;
 
   DBG (DBG_DBG, "do_bindings: socket ()\n");
@@ -2920,7 +2964,7 @@ do_bindings (int *nfds, struct pollfd **fds)
 
 
 static void
-run_standalone (int argc, char **argv)
+run_standalone (char *user)
 {
   struct pollfd *fds = NULL;
   struct pollfd *fdp = NULL;
@@ -2941,13 +2985,13 @@ run_standalone (int argc, char **argv)
 
   if (run_mode != SANED_RUN_DEBUG)
     {
-      if (argc > 2)
+      if (user)
 	{
-	  pwent = getpwnam(argv[2]);
+	  pwent = getpwnam(user);
 
 	  if (pwent == NULL)
 	    {
-	      DBG (DBG_ERR, "FATAL ERROR: user %s not found on system\n", argv[2]);
+	      DBG (DBG_ERR, "FATAL ERROR: user %s not found on system\n", user);
 	      bail_out (1);
 	    }
 
@@ -2978,7 +3022,7 @@ run_standalone (int argc, char **argv)
 
               while (grp->gr_mem[i])
 		{
-                  if (strcmp(grp->gr_mem[i], argv[2]) == 0)
+                  if (strcmp(grp->gr_mem[i], user) == 0)
                     {
                       int need_to_add = 1, j;
 
@@ -3169,57 +3213,79 @@ run_standalone (int argc, char **argv)
 
 
 static void
-run_inetd (int argc, char **argv)
+run_inetd (char __sane_unused__ *sock)
 {
-  int fd = 1;
-  int dave_null;
+  
+  int fd = -1;
 
-  /* Some backends really can't keep their dirty fingers off
-   * stdin/stdout/stderr; we work around them here so they don't
-   * mess up the network dialog and crash the remote net backend.
-   */
-  do
+#ifdef HAVE_SYSTEMD
+  int n;
+
+  n = sd_listen_fds(0);
+
+  if (n > 1) 
     {
-      fd = dup (fd);
-
-      if (fd == -1)
-	{
-	  DBG (DBG_ERR, "run_inetd: duplicating fd failed: %s", strerror (errno));
-	  return;
-	}
-    }
-  while (fd < 3);
-
-  /* Our good'ole friend Dave Null to the rescue */
-  dave_null = open ("/dev/null", O_RDWR);
-  if (dave_null < 0)
-    {
-      DBG (DBG_ERR, "run_inetd: could not open /dev/null: %s", strerror (errno));
+      DBG (DBG_ERR, "run_inetd: Too many file descriptors (sockets) received from systemd!\n");
       return;
     }
 
-  close (STDIN_FILENO);
-  close (STDOUT_FILENO);
-  close (STDERR_FILENO);
+  if (n == 1)
+    {
+    fd = SD_LISTEN_FDS_START + 0;
+    DBG (DBG_INFO, "run_inetd: Using systemd socket %d!\n", fd);
+    }
+#endif
 
-  dup2 (dave_null, STDIN_FILENO);
-  dup2 (dave_null, STDOUT_FILENO);
-  dup2 (dave_null, STDERR_FILENO);
+  if (fd == -1) 
+    {
+      int dave_null;
 
-  close (dave_null);
+      /* Some backends really can't keep their dirty fingers off
+       * stdin/stdout/stderr; we work around them here so they don't
+       * mess up the network dialog and crash the remote net backend
+       * by messing with the inetd socket.
+       * For systemd this not an issue as systemd uses fd >= 3 for the
+       * socket and can even redirect stdout and stderr to syslog.
+       * We can even use this to get the debug logging
+       */
+      do
+        {
+          /* get new fd for the inetd socket */
+          fd = dup (1);
 
-#ifndef HAVE_OS2_H
-  /* Unused in this function */
-  argc = argc;
-  argv = argv;
+          if (fd == -1)
+      	    {
+              DBG (DBG_ERR, "run_inetd: duplicating fd failed: %s", strerror (errno));
+              return;
+            }
+        }
+      while (fd < 3);
 
-#else
+      /* Our good'ole friend Dave Null to the rescue */
+      dave_null = open ("/dev/null", O_RDWR);
+      if (dave_null < 0)
+        {
+          DBG (DBG_ERR, "run_inetd: could not open /dev/null: %s", strerror (errno));
+          return;
+        }
+
+      close (STDIN_FILENO);
+      close (STDOUT_FILENO);
+      close (STDERR_FILENO);
+
+      dup2 (dave_null, STDIN_FILENO);
+      dup2 (dave_null, STDOUT_FILENO);
+      dup2 (dave_null, STDERR_FILENO);
+
+      close (dave_null);
+    }
+#ifdef HAVE_OS2_H
   /* under OS/2, the socket handle is passed as argument on the command
      line; the socket handle is relative to IBM TCP/IP, so a call
      to impsockethandle() is required to add it to the EMX runtime */
-  if (argc == 2)
+  if (sock)
     {
-      fd = _impsockhandle (atoi (argv[1]), 0);
+      fd = _impsockhandle (atoi (sock), 0);
       if (fd == -1)
 	perror ("impsockhandle");
     }
@@ -3228,10 +3294,44 @@ run_inetd (int argc, char **argv)
   handle_connection(fd);
 }
 
+static void usage(char *me, int err)
+{
+  fprintf (stderr,
+       "Usage: %s [OPTIONS]\n\n"
+       " Options:\n\n"
+       "  -a, --alone[=user]	run standalone and fork in background as `user'\n"
+       "  -d, --debug[=level]	run foreground with output to stdout\n"
+       "			and debug level `level' (default is 2)\n"
+       "  -s, --syslog[=level]	run foreground with output to syslog\n"
+       "			and debug level `level' (default is 2)\n"
+       "  -b, --bind=addr	bind address `addr'\n"
+       "  -h, --help		show this help message and exit\n", me);
+
+  exit(err);
+}
+
+static int debug;
+
+static struct option long_options[] =
+{
+/* These options set a flag. */
+  {"help",	no_argument,		0, 'h'},
+  {"alone",	optional_argument,	0, 'a'},
+  {"debug",	optional_argument,	0, 'd'},
+  {"syslog",	optional_argument,	0, 's'},
+  {"bind",	required_argument,	0, 'b'},
+  {0,		0,			0,  0 }
+};
 
 int
 main (int argc, char *argv[])
 {
+  char options[64] = "";
+  char *user = NULL;
+  char *sock = NULL;
+  int c;
+  int long_index = 0;
+
   debug = DBG_WARN;
 
   prog_name = strrchr (argv[0], '/');
@@ -3243,25 +3343,30 @@ main (int argc, char *argv[])
   numchildren = 0;
   run_mode = SANED_RUN_INETD;
 
-  if (argc >= 2)
+  while((c = getopt_long(argc, argv,"ha::d::s::b:", long_options, &long_index )) != -1)
     {
-      if (strncmp (argv[1], "-a", 2) == 0)
+      switch(c) {
+      case 'a':
 	run_mode = SANED_RUN_ALONE;
-      else if (strncmp (argv[1], "-d", 2) == 0)
-	{
-	  run_mode = SANED_RUN_DEBUG;
-	  log_to_syslog = SANE_FALSE;
-	}
-      else if (strncmp (argv[1], "-s", 2) == 0)
+	user = optarg;
+	break;
+      case 'd':
+	log_to_syslog = SANE_FALSE;
+      case 's':
 	run_mode = SANED_RUN_DEBUG;
-    }
-
-  if (run_mode == SANED_RUN_DEBUG)
-    {
-      if (argv[1][2])
-	debug = atoi (argv[1] + 2);
-
-      DBG (DBG_WARN, "main: starting debug mode (level %d)\n", debug);
+	if(optarg)
+	  debug = atoi(optarg);
+	break;
+      case 'b':
+	bind_addr = optarg;
+	break;
+      case 'h':
+	usage(argv[0], EXIT_SUCCESS);
+	break;
+      default:
+	usage(argv[0], EXIT_FAILURE);
+	break;
+      }
     }
 
   if (log_to_syslog)
@@ -3276,24 +3381,41 @@ main (int argc, char *argv[])
   wire.io.read = read;
   wire.io.write = write;
 
-/* define the version string depending on which network code is used */
 #ifdef SANED_USES_AF_INDEP
+  strcat(options, "AF-indep");
 # ifdef ENABLE_IPV6
-  DBG (DBG_WARN, "saned (AF-indep+IPv6) from %s starting up\n", PACKAGE_STRING);
-# else
-  DBG (DBG_WARN, "saned (AF-indep) from %s starting up\n", PACKAGE_STRING);
-# endif /* ENABLE_IPV6 */
+  strcat(options, "+IPv6");
+#endif
 #else
-  DBG (DBG_WARN, "saned from %s ready\n", PACKAGE_STRING);
-#endif /* SANED_USES_AF_INDEP */
-
-  if ((run_mode == SANED_RUN_ALONE) || (run_mode == SANED_RUN_DEBUG))
+  strcat(options, "IPv4 only");
+#endif
+#ifdef HAVE_SYSTEMD
+  if (sd_listen_fds(0) > 0)
     {
-      run_standalone(argc, argv);
+      strcat(options, "+systemd");
+    }
+#endif
+
+  if (strlen(options) > 0)
+    {
+      DBG (DBG_WARN, "saned (%s) from %s starting up\n", options, PACKAGE_STRING);
     }
   else
     {
-      run_inetd(argc, argv);
+      DBG (DBG_WARN, "saned from %s ready\n", PACKAGE_STRING);
+    }
+
+  if ((run_mode == SANED_RUN_ALONE) || (run_mode == SANED_RUN_DEBUG))
+    {
+      run_standalone(user);
+    }
+  else
+    {
+#ifdef HAVE_OS2_H
+      if (argc == 2)
+	sock = argv[1];
+#endif
+      run_inetd(sock);
     }
 
   DBG (DBG_WARN, "saned exiting\n");
