@@ -3,6 +3,7 @@
 
    Uses the SANE library.
    Copyright (C) 2002 Frank Zago (sane at zago dot net)
+   Copyright (C) 2013 Stphane Voltz <stef.dev@free.fr> : sane_get_devices test
 
    This file is part of the SANE package.
 
@@ -22,7 +23,7 @@
    MA 02111-1307, USA.
 */
 
-#define BUILD 18				/* 2003-02-22 */
+#define BUILD 19				/* 2013-03-29 */
 
 #include "../include/sane/config.h"
 
@@ -45,7 +46,9 @@
 static struct option basic_options[] = {
 	{"device-name", required_argument, NULL, 'd'},
 	{"level", required_argument, NULL, 'l'},
+	{"scan", NULL, NULL, 's'},
 	{"recursion", required_argument, NULL, 'r'},
+	{"get-devices", required_argument, NULL, 'g'},
 	{"help", 0, NULL, 'h'}
 };
 
@@ -71,6 +74,7 @@ long int checks_done = 0;
 #endif
 
 int test_level;
+int verbose_level;
 
 /* Maybe add that to sane.h */
 #define SANE_OPTION_IS_GETTABLE(cap)	(((cap) & (SANE_CAP_SOFT_DETECT | SANE_CAP_INACTIVE)) == SANE_CAP_SOFT_DETECT)
@@ -398,9 +402,15 @@ test_options (SANE_Device * device, int can_do_recursive)
 			continue;
 		}
 
+		if(verbose_level) {
+			printf("checking option ""%s""\n",opt->title);
+		}
+
 		if (opt->type == SANE_TYPE_GROUP) {
 			check(INF, (opt->name == NULL || *opt->name == 0),
 				  "option [%d, %s] has a name", option_num, opt->name);
+			check(ERR, (!SANE_OPTION_IS_SETTABLE (opt->cap)),
+				  "option [%d, %s], group option is settable", option_num, opt->name);
 		} else {
 			if (option_num == 0) {
 				check(ERR, (opt->name != NULL && *opt->name ==0),
@@ -1125,7 +1135,7 @@ static void test_scan(SANE_Handle device)
 	unsigned char *image = NULL;
 	SANE_Parameters params;
 	size_t to_read;
-	SANE_Int len;
+	SANE_Int len=0;
 	int ask_len;
 	int rc;
 	int fd;
@@ -1555,12 +1565,120 @@ static void test_scans(SANE_Device * device)
 	}
 }
 
+/** test sane_get_devices
+ * test sane_get_device function, if time is greter than 0,
+ * loop to let tester plug/unplug device to check for correct
+ * hotplug detection
+ * @param device_list device list to fill
+ * @param time time to loop
+ * @return 0 on success
+ */
+static int test_get_devices(const SANE_Device ***device_list, int time)
+{
+int loop=0;
+int i;
+const SANE_Device *dev;
+SANE_Status status;
+
+	status = sane_get_devices (device_list, SANE_TRUE);
+	check(FATAL, (status == SANE_STATUS_GOOD),
+		  "sane_get_devices() failed (%s)", sane_strstatus (status));
+
+	/* Verify that the SANE doc (or tstbackend) is up to date */
+	for (i=0; (*device_list)[i] != NULL; i++) {
+
+		dev = (*device_list)[i];
+
+		check(FATAL, (dev->name != NULL), "device name is NULL");
+		check(FATAL, (dev->vendor != NULL), "device vendor is NULL");
+		check(FATAL, (dev->type != NULL), "device type is NULL");
+		check(FATAL, (dev->model != NULL), "device model is NULL");
+		
+		check(INF, ((strcmp(dev->type, "flatbed scanner") == 0) ||
+					(strcmp(dev->type, "frame grabber") == 0) ||
+					(strcmp(dev->type, "handheld scanner") == 0) ||
+					(strcmp(dev->type, "still camera") == 0) ||
+					(strcmp(dev->type, "video camera") == 0) ||
+					(strcmp(dev->type, "virtual device") == 0) ||
+					(strcmp(dev->type, "film scanner") == 0) ||
+					(strcmp(dev->type, "multi-function peripheral") == 0) ||
+					(strcmp(dev->type, "sheetfed scanner") == 0)),
+					"unknown device type [%s]. Update SANE doc section \"Type Strings\"", dev->type);
+
+		check(INF, (
+					(strcmp(dev->vendor, "AGFA") == 0) ||
+					(strcmp(dev->vendor, "Abaton") == 0) ||
+					(strcmp(dev->vendor, "Acer") == 0) ||
+					(strcmp(dev->vendor, "Apple") == 0) ||
+					(strcmp(dev->vendor, "Artec") == 0) ||
+					(strcmp(dev->vendor, "Avision") == 0) ||
+					(strcmp(dev->vendor, "CANON") == 0) ||
+					(strcmp(dev->vendor, "Connectix") == 0) ||
+					(strcmp(dev->vendor, "Epson") == 0) ||
+					(strcmp(dev->vendor, "Fujitsu") == 0) ||
+					(strcmp(dev->vendor, "Gphoto2") == 0) ||
+					(strcmp(dev->vendor, "Hewlett-Packard") == 0) ||
+					(strcmp(dev->vendor, "IBM") == 0) ||
+					(strcmp(dev->vendor, "Kodak") == 0) ||
+                                        (strcmp(dev->vendor, "Lexmark") == 0) ||
+					(strcmp(dev->vendor, "Logitech") == 0) ||
+					(strcmp(dev->vendor, "Microtek") == 0) ||
+					(strcmp(dev->vendor, "Minolta") == 0) ||
+					(strcmp(dev->vendor, "Mitsubishi") == 0) ||
+					(strcmp(dev->vendor, "Mustek") == 0) ||
+					(strcmp(dev->vendor, "NEC") == 0) ||
+					(strcmp(dev->vendor, "Nikon") == 0) ||
+					(strcmp(dev->vendor, "Noname") == 0) ||
+					(strcmp(dev->vendor, "Plustek") == 0) ||
+					(strcmp(dev->vendor, "Polaroid") == 0) ||
+					(strcmp(dev->vendor, "Relisys") == 0) ||
+					(strcmp(dev->vendor, "Ricoh") == 0) ||
+					(strcmp(dev->vendor, "Sharp") == 0) ||
+					(strcmp(dev->vendor, "Siemens") == 0) ||
+					(strcmp(dev->vendor, "Tamarack") == 0) ||
+					(strcmp(dev->vendor, "UMAX") == 0)),
+			  "unknown device vendor [%s]. Update SANE doc section \"Vendor Strings\"", dev->vendor);
+	}
+
+	/* loop on detecting device to let time to plug/unplug scanners */
+	while(loop<time) {
+		/* print and free detected device list */
+		check(MSG, 0, "DETECTED DEVICES:");
+		for (i=0; (*device_list)[i] != NULL; i++) {
+			dev = (*device_list)[i];
+			check(MSG, 0, "\t%s:%s %s:%s", dev->vendor, dev->name, dev->type, dev->model);
+		}
+		if(i==0) {
+			check(MSG, 0, "\tnone...");
+		}
+		sleep(1);
+		(*device_list) = NULL;
+		status = sane_get_devices (device_list, SANE_TRUE);
+		check(FATAL, (status == SANE_STATUS_GOOD),
+		  "sane_get_devices() failed (%s)", sane_strstatus (status));
+		loop++;
+	}
+	return 0;
+}
+
+/** test test_default
+ * test by scanning using default values
+ * @param device device to use for the scan
+ */
+static void test_default(SANE_Device * device)
+{
+	test_scan(device);
+}
+
 static void usage(const char *execname)
 {
-	printf("Usage: %s [-d backend_name] [-l test_level] [-r recursion_level]\n", execname);
+	printf("Usage: %s [-d backend_name] [-l test_level] [-s] [-r recursion_level] [-g time (s)]\n", execname);
+	printf("\t-v\tverbose level\n");
 	printf("\t-d\tbackend name\n");
 	printf("\t-l\tlevel of testing (0=some, 1=0+options, 2=1+scans, 3=longest tests)\n");
+	printf("\t-s\tdo a scan during open/close tests\n");
 	printf("\t-r\trecursion level for option testing (the higher, the longer)\n");
+	printf("\t-g\ttime to loop on sane_get_devices function to test scannet hotplug detection (time is in seconds).\n");
 }
 
 int
@@ -1574,24 +1692,31 @@ main (int argc, char **argv)
 	int index;
 	int i;
 	const SANE_Device **device_list;
-	const SANE_Device *dev;
 	int rc;
 	int recursion_level;
+	int time;
+	int default_scan;
 
 	printf("tstbackend, Copyright (C) 2002 Frank Zago\n");
 	printf("tstbackend comes with ABSOLUTELY NO WARRANTY\n");
-    printf("This is free software, and you are welcome to redistribute it\n");
-    printf("under certain conditions. See COPYING file for details\n\n");
+	printf("This is free software, and you are welcome to redistribute it\n");
+	printf("under certain conditions. See COPYING file for details\n\n");
 	printf("This is tstbackend build %d\n\n", BUILD);
 
 	/* Read the command line options. */
 	opterr = 0;
 	recursion_level = 5;		/* 5 levels or recursion should be enough */
-	test_level = 0;				/* basic tests only */
+	test_level = 0;			/* basic tests only */
+	time = 0;			/* no get devices loop */
+	default_scan = 0;
 
-	while ((ch = getopt_long (argc, argv, "-d:l:r:h", basic_options,
+	while ((ch = getopt_long (argc, argv, "-v:d:l:r:g:h:s", basic_options,
 							  &index)) != EOF) {
 		switch(ch) {
+		case 'v':
+			verbose_level = atoi(optarg);
+			break;
+
 		case 'd':
 			devname = strdup(optarg);
 			break;
@@ -1604,10 +1729,18 @@ main (int argc, char **argv)
 			}
 			break;
 
+		case 's':
+			default_scan = 1;
+			break;
+
 		case 'r':
 			recursion_level = atoi(optarg);
 			break;
 
+		case 'g':
+			time = atoi(optarg);
+			break;
+
 		case 'h':
 			usage(argv[0]);
 			return(0);
@@ -1678,69 +1811,8 @@ main (int argc, char **argv)
 		  "sane_init failed with %s", sane_strstatus (status));
   
 	/* Check the device list */
-	status = sane_get_devices (&device_list, SANE_TRUE);
-	check(FATAL, (status == SANE_STATUS_GOOD),
-		  "sane_get_devices() failed (%s)", sane_strstatus (status));
-
-	/* Verify that the SANE doc (or tstbackend) is up to date */
-	for (i=0; device_list[i] != NULL; i++) {
-
-		dev = device_list[i];
-
-		check(FATAL, (dev->name != NULL),
-			  "device name is NULL");
-		check(FATAL, (dev->vendor != NULL),
-			  "device vendor is NULL");
-		check(FATAL, (dev->type != NULL),
-			  "device type is NULL");
-		check(FATAL, (dev->model != NULL),
-			  "device model is NULL");
-		
-		check(INF, ((strcmp(dev->type, "flatbed scanner") == 0) ||
-					(strcmp(dev->type, "frame grabber") == 0) ||
-					(strcmp(dev->type, "handheld scanner") == 0) ||
-					(strcmp(dev->type, "still camera") == 0) ||
-					(strcmp(dev->type, "video camera") == 0) ||
-					(strcmp(dev->type, "virtual device") == 0) ||
-					(strcmp(dev->type, "film scanner") == 0) ||
-					(strcmp(dev->type, "multi-function peripheral") == 0) ||
-					(strcmp(dev->type, "sheetfed scanner") == 0)),
-					"unknown device type [%s]. Update SANE doc section \"Type Strings\"", dev->type);
-
-		check(INF, (
-					(strcmp(dev->vendor, "AGFA") == 0) ||
-					(strcmp(dev->vendor, "Abaton") == 0) ||  
-					(strcmp(dev->vendor, "Acer") == 0) ||
-					(strcmp(dev->vendor, "Apple") == 0) ||
-					(strcmp(dev->vendor, "Artec") == 0) ||
-					(strcmp(dev->vendor, "Avision") == 0) ||
-					(strcmp(dev->vendor, "CANON") == 0) ||
-					(strcmp(dev->vendor, "Connectix") == 0) ||
-					(strcmp(dev->vendor, "Epson") == 0) ||
-					(strcmp(dev->vendor, "Fujitsu") == 0) ||
-					(strcmp(dev->vendor, "Gphoto2") == 0) ||
-					(strcmp(dev->vendor, "Hewlett-Packard") == 0) ||
-					(strcmp(dev->vendor, "IBM") == 0) ||
-					(strcmp(dev->vendor, "Kodak") == 0) ||
-                                        (strcmp(dev->vendor, "Lexmark") == 0) ||
-					(strcmp(dev->vendor, "Logitech") == 0) ||
-					(strcmp(dev->vendor, "Microtek") == 0) ||
-					(strcmp(dev->vendor, "Minolta") == 0) ||
-					(strcmp(dev->vendor, "Mitsubishi") == 0) ||
-					(strcmp(dev->vendor, "Mustek") == 0) ||
-					(strcmp(dev->vendor, "NEC") == 0) ||
-					(strcmp(dev->vendor, "Nikon") == 0) ||
-					(strcmp(dev->vendor, "Noname") == 0) ||
-					(strcmp(dev->vendor, "Plustek") == 0) ||
-					(strcmp(dev->vendor, "Polaroid") == 0) ||
-					(strcmp(dev->vendor, "Relisys") == 0) ||
-					(strcmp(dev->vendor, "Ricoh") == 0) ||
-					(strcmp(dev->vendor, "Sharp") == 0) ||
-					(strcmp(dev->vendor, "Siemens") == 0) ||
-					(strcmp(dev->vendor, "Tamarack") == 0) ||
-					(strcmp(dev->vendor, "UMAX") == 0)),
-			  "unknown device vendor [%s]. Update SANE doc section \"Vendor Strings\"", dev->vendor);
-	}
+	rc = test_get_devices(&device_list, time);
+	if (rc) goto the_exit;
 
 	if (!devname) {
 		/* If no device name was specified explicitly, we look at the
@@ -1769,6 +1841,10 @@ main (int argc, char **argv)
 		rc = check(ERR, (status == SANE_STATUS_GOOD),
 				   "sane_open failed with %s for device %s", sane_strstatus (status), devname);
 		if (!rc) goto the_exit;
+
+		if (default_scan) {
+			test_default (device);
+		}
 		sane_close (device);
 	}
 
