@@ -94,6 +94,9 @@
 #   define __GNUC__ 2
 #  endif
 #  include <sys/io.h>
+#  ifndef SANE_HAVE_SYS_IO_H_WITH_INB_OUTB
+#   define IO_SUPPORT_MISSING
+#  endif
 #  if defined (__ICC) && __ICC >= 700
 #   undef __GNUC__
 #  elif defined(__ICC) && defined(HAVE_ASM_IO_H)
@@ -123,9 +126,7 @@ inb( u_long port )
 #elif defined(HAVE_LIBIEEE1284)
 # include <ieee1284.h>
 #else
-# if defined(__GNUC__)
-#  warning "No I/O support for this architecture!"
-# endif
+# pragma message "No I/O support for this architecture!"
 # define IO_SUPPORT_MISSING
 #endif
 
@@ -694,7 +695,7 @@ pp_init( void )
 #endif
 
 	if( first_time == SANE_FALSE ) {
-		DBG( 5, "pp_init: already initalized\n" );
+		DBG( 5, "pp_init: already initialized\n" );
 		return SANE_STATUS_GOOD;
     }
 
